# --*-PERL-*--
#####################################################################
#
# Copyright 1996, CyberCash Inc. All rights reserved.
# Written by Brian Boesch and Judy Grass
# Modified by Gene Gotimer
# and then by Judy Grass for MCK
#
# These routines are intended to be used exclusively with the 
# CyberCash secure payment system.
#
# RCSID: $Id: CCMckDirectLib3_2.pm,v 1.23.2.3 2000/01/24 22:47:45 gotimer Exp $
#
# Do not redistribute without written permission from CyberCash Inc.
######################################################################

# This CCMckDirectLib3_2.pm is part of the mck-3.2.0.6-i586-pc-linux-gnulibc2.1 distribution.

package CCMckDirectLib3_2;

require Exporter;
@ISA = (Exporter);
@EXPORT = qw();
@EXPORT_OK = qw(doDirectPayment SendCC2_1Server SendCCServer CCSocketSend );

use CCMckLib3_2 qw(%Config Trim 
    URLdecodeForm
	genURLencodedForm
	BuildBlock
    CCDebug CCDebug2 CCError
	);

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr $E_ERROR

	$E_BadMethod
	$E_ConnectFailed
	$E_DecryptFailed
	$E_EncryptFailed
	$E_Failed_HTTP
	$E_GoAway
	$E_No_Config
	$E_No_MAC
	$E_No_Message
	$E_No_POP
	$E_No_Skey
	$E_No_SSL 
	$E_Not_HTTP
	$E_SignatureFailed
	$E_SocketFailed
	$E_SyswriteFailed
	$E_TimedOut
   );

# UNIX or NT version
$OS = 'UNIX';

use Socket;
use Time::Local;

$TMPDIR = "/tmp/";
$FILESEP = '/';
 
if ($OS eq 'NT') {
    $TMPDIR = $ENV{'TEMP'} || "c:\\temp";
 
    unless ($TMPDIR =~ /.*\\$/) {
        $TMPDIR = "$TMPDIR\\";
    }
 
    $FILESEP = '\\';
}

$MCKencrypt = "/home/httpd/test-store/cgi-bin/mck-cgi/MCKencrypt";
$MCKdecrypt = "/home/httpd/test-store/cgi-bin/mck-cgi/MCKdecrypt";

# Socket Errors


sub doDirectPayment
{
	my ($paymentURL,  $argNVListref) = @_;
	my (%paymentArgs, $token);
	my (%MO, %CPI, %MF, %args, %tokenList, %result);

		# These are used for logging if a POP is not returned...
	my ($orderId, $timestamp);
	my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
	
	%paymentArgs = %$argNVListref;
	
		# for templating after the payment completes
	%tokenList = %paymentArgs;

	#We build the arguments then call CCSend
	$orderId = $paymentArgs{'mo.order-id'};

	# Build blocks
    %MO = &BuildBlock("mo", %paymentArgs);
    %CPI = &BuildBlock("cpi", %paymentArgs);
    %MF = &BuildBlock("mf", %paymentArgs);

	$args{'MO'} = genURLencodedForm(%MO);
	$tokenList{'MO'} = $args{'MO'};
	
	$args{'CPI'} = genURLencodedForm(%CPI);
	$tokenList{'CPI'} = $args{'CPI'};

	# No reason to send MF... response comes back here!
	# Args will be used for templating, so make sure *everything* is there

	$tokenList{'MF'} = genURLencodedForm(%MF);

	%result = &SendCCServer($paymentURL, %args);

	# Result will have at least:
	#	'MStatus'
	#	'MErrLoc'
	#	'MErrMsg'
	# since SendCCServer will generate these for any message that
	# does not naturally return them
	#
	#  Any "normal" return will contain a POP block as well.
	#  Load that into the tolken list
	#  *AND* if the POP is not present, generate an error POP

	if (defined($result{'POP'})) {
		# Phew! at least we got a response
		# Set up for templating

		$POPstr = $result{'POP'};

		$tokenList{'POP'} = $POPstr;
		%POP = URLdecodeForm($POPstr);

	}
	else {
		#*sigh*  Build a false POP from the status info, etc.
		$POP{'pop.status'} = $result{'MStatus'};
		$POP{'pop.error-code'} = $E_NoPOP;
		
		#build an error message...
		$tstr = "Location: ". $result{'MErrLoc'} 
			. "; Message: " . $result{'MErrMsg'};

		$POP{'pop.error-message'} = $tstr;

			# These are needed for logging..
		$POP{'pop.order-id'} = $orderId;  # need this for logging

		($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time());
		
		#According to man gmtime... $year is tm.tm_year: years since 1900....
		# This is correct for Y2K

		$POP{'TEST'} = "TESTING";

		$POP{'pop.sale-date'} = sprintf("%04d%02d%02d%02d%02d%02d.000",
            $year+1900,$mon+1,$mday,$hour,$min,$sec);

		$tokenList {'POP'} = genURLencodedForm(%POP);
	} 

	# load everything in the POP into the templating array 
	# and into result array
	foreach $key (keys (%POP)) {
		$tokenList{$key} = $POP{$key};
		$result{$key} = $POP{$key};
	}

	$tokenList{'error-code'} = $POP{'pop.error-code'};
	$tokenList{'error-message'} = $POP{'pop.error-message'};
	$tokenList{'status'} = $POP{'pop.status'};

	# Also need to build a failureAnchor....
	if (defined($MO{'mo.failure-url'})) {
		$tokenList{'failAnchor'} = 
			"<a href=\"" . $MO{'mo.failure-url'} . "\"> Additional Information </a> ";
	}
	else {
		$tokenList{'failAnchor'} = " ";
	}
	
	# And build the Manual Notification parameters
	# Use only the POP and MF... nothing outside blocks.

	$tstr = "\t<input type=hidden name=\"POP\" value=\"";
	$tstr .= $result{'POP'};
	$tstr .= "\">\r\n";

	$tstr .= "\t<input type=hidden name=\"MF\" value=\"";
	$tstr .= $tokenList{'MF'};
	$tstr .= "\">\r\n";

	$tokenList{'MANUAL_NOTIFICATION'} = $tstr;

	## Tokens can contain card data...
	## Don't dump this stuff except when testing with fake card data

	##&CCDebug2(3, "Token List is: \n");
	##foreach $key (keys (%tokenList)) {
	##	$tstr = "$key => " . $tokenList{$key} . "\n";
	##	&CCDebug2(3, "$tstr");
	##}
	##&CCDebug2(3, "END TOKEN LIST\n");

	#This gets us (almost?) everything in the token list that the 
	# wrapper would have had...
	
	return (\%POP, \%tokenList);    # all is well.. return references
}

##########################################################
# SendCC2_1Server(operation,
#      attribute, value,
#      attribute2, value2,
#      ...);
# 
# Returns set of attribute values appropriate for an assoc array
#
# This takes the old sendmserver format and generates a message to the wrapper
# CR2.1 API interpreter

sub SendCC2_1Server {
	# Get the url and name-value pairs passed as parameters
    my($operation, @pairs) = @_;

	my($url, %result);

	$url = $Config{"CCPS_HOST"} . "cr21api.cgi/$operation";

	&CCDebug2(7, "Url is $url\n");

	%result = &SendCCServer($url, @pairs);
	
	return %result;
}


##########################################################
# SendCCServer(url,
#      attribute, value,
#      attribute2, value2,
#      ...);
# 
# Returns set of attribute values appropriate for an assoc array
#
sub SendCCServer {

	# Get the url and name-value pairs passed as parameters
    my($url, @pairs) = @_;

	my($method, $ccaddr, $timeout);
	my($CCMerchantId, $CCServerHost, $CCServerPort);
    my($encoded_pairs);
    my($pairs_length,$message);
	my($status, $response);
	my($sessionKey, $MAC, $encryptedMsg);
	my(%args) = ();
    my(%result) = ();

	# Strip the method string from the URL...
	$url = &Trim($url);    #make it tidy
	$url =~ s#^([\w]+)://##;   #strip off the method
	$method = $1;

	if ($method ne "http") {
		# GONG! we don't do https or ftp or anything else!
		if ($method eq "https") {  # special message for this one
			return(&FailureResponse($E_No_SSL, ""));
		}
		else {
			return(&FailureResponse($E_BadMethod, ""));
		}
	}

    $CCMerchantId = $Config{"CYBERCASH_ID"};

	# get the host and port from the URL, or from the HTTP_PROXY
	if (defined($Config{"HTTP_PROXY_HOST"})
		&& ($Config{"HTTP_PROXY_HOST"} ne "")) 
	{
    	$CCServerHost = $Config{"HTTP_PROXY_HOST"};
    	$CCServerPort = $Config{"HTTP_PROXY_PORT"};

		#Proxy url is .... http://host:port/yada/yada
		$url = "http://$url";
	}
	else {
		# Get host and port from URL
		# grab everything before the first slash
		$url =~ s#([^/]+)/##;
		$ccaddr = $1;

		($CCServerHost, $CCServerPort) = split(/:/, $ccaddr, 2);
		$url = "/$url";
	}

	if ((!defined($CCServerPort)) || ($CCServerPort eq "")) {
		$CCServerPort = "80";  # http: default
	}

	# Token code can contain senitive items,
    # Don't dump this except maybe when testing with fake data.

	##&CCDebug2(9,"\n== Entering SendCCServer ==\n");
	##&CCDebug2(9,"POST to host: $CCServerHost, port: $CCServerPort\n");
	##&CCDebug2(9,"secret: $CCMerchantId, command: $url\n\n");
	##	
	##&CCDebug2(9,"## Name-Value pairs to send ##\n");
	##
	##my($i,$j);
	##for ($i=0; $i<@pairs; $i+=2) {
	##   $j = $i + 1;
	##  &CCDebug2(9,"$pairs[$i] = [[$pairs[$j]]]\n");
	##}
	##&CCDebug2(9,"\n");

	# turn the name-value pairs into a url encoded message
    $encoded_pairs = &genURLencodedForm(@pairs);

	##&CCDebug2(9,"Encoded name-value pairs: $encoded_pairs\n");
	
	#Encrypt the encoded pairs....
	($status, $sessionKey, $MAC, $encryptedMsg) = &CCEncrypt($encoded_pairs);

    if ($status != $E_NoErr) {
        return (&FailureResponse($status, ""));
    }
	
	#Build the message
	$args{'session-key'} = $sessionKey;
	$args{'message'} = $encryptedMsg;
	$args{'mac'} = $MAC;

	$encryptedPairs = &genURLencodedForm(%args);
	

	# Create the message before opening the socket
	# we want the socket code to be as isolated as possible, 
	# since that is the code that is tricky to port... 

    $pairs_length = length($encryptedPairs);
    $message = <<"END";
POST $url/$CCMerchantId HTTP/1.0
User-Agent: CCMCK-$version
Content-type: application/x-www-form-urlencoded
Content-length: $pairs_length

$encryptedPairs
END

	&CCDebug2(7, "Encoded/ Encrypted name-value pairs length = $pairs_length\n");
	&CCDebug2(7, "Message:\n$message\n");

	# We've built the message... now send it.
	# Actual Socket stuff is isolated here!
	$timeout = $Config{"TIMEOUT"};

	($status, $response) 
		= &CCSocketSend($CCServerHost, $CCServerPort, $message, $timeout);

	# Look at status and generate an appropriate dummy
	# message if there has been a socket failure of some kind

	if ($status != $E_NoErr) {
		return (&FailureResponse($status, $response));
	}

	#Trim off any white space...
	$response = &Trim($response);

	# Response is encrypted... decrypt it...
	($status, $response) = &CCDecrypt($response);

    if ($status != $E_NoErr) {
        return (&FailureResponse($status, ""));
    }

	# decode the url encoded response, and send back the result
	# as an associative array (%result)

    %result = &URLdecodeForm($response);

	## Don't dump raw message data unless testing, since it can
	## contain account info.
	##
	##&CCDebug2(9, "## Decoded response Name-Value pairs ##\n");
	##foreach (keys %result) {
	##   	&CCDebug2(9, "$_ = [[$result{$_}]]\n");
	##}
	##&CCDebug2(9, "\n\n== Exiting SendCCServer ==\n\n\n");

    return %result;
}

use IPC::Open2;
sub CCEncrypt 
{
  my($message) = @_;

  &CCDebug2(6, "${indent}##### in CCEncrypt ######\n");

  # open the program for encryption
  unless (open2(\*CIPHER, \*PLAINTEXT, $MCKencrypt)) {
    return($E_EncryptFailed, &MCKGetErrorMessage($E_EncryptFailed));
  }

  my $key = $Config{"MERCHANT_KEY"};

  # We do not log the encryption input because it may contain
  # sensitive data: like credit card or checking account numbers.  If
  # you need to see the input, uncomment these lines and set the debug
  # level as needed

  ##&CCDebug2(6, "${indent} key =$key\n");
  ##&CCDebug2(6, "${indent} encrypting:\n$message\n");

  print PLAINTEXT "$key\r\n$message\r\n\032";
  close(PLAINTEXT);

  # first line is the mac
  my $mac = <CIPHER>;
  $mac =~ s/[\cM\cJ]{0,2}$//o;

  # second line is the session-key
  my $sessionkey = <CIPHER>;
  $sessionkey =~ s/[\cM\cJ]{0,2}$//o;

  # the rest is the encrypted message
  my $encrypted = '';
  while (<CIPHER>) {
    s/[\cM\cJ]{0,2}$//o;
    $encrypted .= $_;
  }
  close(CIPHER);

  # if there are no alphanumerics returned, it didn't work
  unless ($mac =~ /\w/o) {
    &CCError(&MCKGetErrorMessage($E_No_MAC));
    return($E_EncryptFailed, $sessionkey, $mac, $encrypted);
  }

  # if the response starts with a '###', there has been an error
  if ($mac =~ /^\s*\#\#\#/o) {
    &CCError("$mac\n");
    return($E_EncryptFailed, $mac);
  }

  # if there is no session-key returned, it didn't work
  unless ($sessionkey) {
    &CCError(&MCKGetErrorMessage($E_No_Skey));
    return($E_EncryptFailed, $sessionkey, $mac, $encrypted);
  }

  # if there is no encrypted message returned, it didn't work
  unless ($encrypted) {
    &CCError(&MCKGetErrorMessage($E_No_Message));
    return($E_EncryptFailed, $sessionkey, $mac, $encrypted);
  }

  return(0, $sessionkey, $mac, $encrypted);
}

sub CCDecrypt 
{
  my($raw) = @_;

  # Message has to be unwrapped to get the pieces
  &CCDebug2(6, "${indent}##### in CCDecrypt ######\n");

  %fields = &URLdecodeForm($raw);

  # Response is not encrypted if there is no session key Implies that
  # there is a configuration problem either in the MCK or the wrapper
  # that prevented the merchant from being recognized

  unless (defined $fields{'session-key'}) {
    return(0, $raw);
  }

  my $key = $Config{"MERCHANT_KEY"};

  my $sessionkey = $fields{'session-key'};
  my $mac = $fields{'mac'};
  my $message = $fields{'message'};

  # We do not log the decryption input because it may contain
  # sensitive data: like credit card or checking account numbers.  If
  # you need to see the input, uncomment these lines and set the debug
  # level as needed

  ##&CCDebug2(6, "${indent} key =$key\n");
  ##&CCDebug2(6, "${indent} decrypting:\n$message\n");
  ##&CCDebug2(6, "${indent} MAC:\n$mac\n");
  ##&CCDebug2(6, "${indent} skey:\n$sessionkey\n");

  # decrypt the message
  unless (open2(\*PLAINTEXT, \*CIPHER, $MCKdecrypt)) {
    return($E_DecryptFailed, &MCKGetErrorMessage($E_DecryptFailed));
  }

  print CIPHER "$key\r\n$sessionkey\r\n$mac\r\n$message\r\n\032";
  close(CIPHER);

  my $decrypted = <PLAINTEXT>;
  $decrypted =~ s/[\cM\cJ]{0,2}$//o;

  close(PLAINTEXT);

  # if there are no alphanumerics returned, it didn't work
  unless ($decrypted =~ /\w/o) {
    &CCError(&MCKGetErrorMessage($E_No_Message));
    return($E_DecryptFailed, &MCKGetErrorMessage($E_No_Message));
  }

  # if the response starts with a '#', there has been an error
  if ($decrypted =~ /^\s*\#/o) {
    &CCError("$decrypted\n");
    return($E_DecryptFailed, $decrypted);
  }

  ## &CCDebug2(9, "Decrypted response is $decrypted\n");
  return(0, $decrypted);
}

# Here is the code that opens a socket and squirts down a message.
# returns a status code and the "undigested" reponse string

sub CCSocketSend
{
	my ($CCServerHost, $CCServerPort, $message, $timeout) = @_;


    my($cc_server_addr,@cc_server_addrs,$cc_server_socket);
    my($name,$aliases,$protocol,$addrtype,$length);
    my($message_length,$write_result);
    my($TimedOut,$socket_data,@socket_data);
	my($line, $isStatusLine);
	my($errcode) = 0; #no error
	my($code);
	my($oldFH);

	# set up the socket to talk to the payment server
	if ($OS eq 'UNIX') {
		$SOCK_STREAM = SOCK_STREAM;
		$AF_INET = AF_INET;
	} else {
		$SOCK_STREAM = 1;
		$AF_INET = 2;
	}

	# find the TCP protocol information
    ($name,$aliases,$protocol) = getprotobyname('tcp');

	# find the address for the payment server
   if ($CCServerHost =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
      $cc_server_addr = pack("C4", $1, $2, $3, $4);
   } else {
      ($name,$aliases,$addrtype,$length,@cc_server_addrs) 
			= gethostbyname($CCServerHost);
      $cc_server_addr = $cc_server_addrs[0];
   }

	# Put the socket info into the correct format for the connect

	$cc_server_socket 
		= sockaddr_in($CCServerPort,$cc_server_addr);
	
	# make the socket filehandle and connect to the socket
	# if any fail, reply and return with an appropriate error message
	# "S" is the socket filehandle

    socket(S,$AF_INET,$SOCK_STREAM,$protocol) 
		|| return ($E_SocketFailed, "");

	&CCDebug2(9,  "\nOpened socket.\n");

    connect(S,$cc_server_socket) 
		|| return ($E_ConnectFailed, "" );

	&CCDebug2(9, "Connected to socket.\n\n");

    if ($OS eq 'UNIX') { # alarms are not implemented in NT Perl
			# set up a handler for response timeouts
		$SIG{'ALRM'} = \&Timeout;

			# SIGALRM will now call the &CCMckDirectLib3_2::Timeout routine
			# set a flag so we can see if we timed out
		$TimedOut = 0;

			# set the timeout value for sending a message    
			# WARNING: The payment server may wait up to 90 seconds for
			#          a response from the CyberCash server. If you don't 
			#          wait at least 90 seconds, you may be giving up 
			#          too early. Be patient.
		$timeout = 120;
		alarm($timeout);
    }

    $message_length = length($message);

	&CCDebug2(9, "Message length = $message_length\n");

		# force flush the output buffer
    $oldFH = select(S); $| = 1; print ''; select($oldFH);

		# send the message over the socket
    if ($OS eq 'UNIX') {
			# $write_result will be undef if there was an error
		$write_result = syswrite(S,$message,$message_length);
		unless ( defined($write_result) ) 
		{ 
			return ($E_SyswriteFailed, "");
		}

    } else {

		print S $message;

    }

		# reinstate buffered I/O
    $| = 0;

	&CCDebug2(9, "Wrote message to socket.\n");
	&CCDebug2(9, "Reading response.\n");

    if ($OS eq 'UNIX') {
		# reset the alarm while we wait for a response
		alarm($timeout);
    }

		# read the socket line-by-line
		# First line is special... status line
	$isStatusLine = 1;
	
    while (<S>) {

		if ($OS eq 'UNIX') {
			# Results filled in by calling routine
			if ($TimedOut) {  return ($E_TimedOut, "") ; }
		}

		$line = $_;
		
		#If this is the first line, check the status
		if ($isStatusLine == 1) {
			#Is this a happy status?
			if ($line =~ /^HTTP\//i ) 
			{
				# looks like a good response
				if ($line =~ /[ \t]([\d]+ .*)/ ) {

					$code = $1;
				
					if ($code !~/^2/) {
						return ($E_Failed_HTTP, $code);
					} 
				}
				else {
					return ($E_NotHTTP, "");
				}
			}
			else {
				return ($E_NotHTTP, "");
			}

			
			$isStatusLine = 0;
		}

		# if we are told to go away, then we should do just that

		if ($line =~ /x-go-away-message/i) { 
			return ( $E_GoAway, "");
		}

		# read until a blank line
		last if ($line =~ /^\s*$/);

		if ($OS eq 'UNIX') { # reset the alarm for the next line
		    alarm($timeout);
		}

    }

    if ($OS eq 'UNIX') { # reset the alarm for the all-at-once read
		alarm($timeout);
    }

	# we want to read all the rest of the data, all at once,
	# then convert to a single scalar value

    @socket_data = <S>;
    $socket_data = join('',@socket_data);

	##&CCDebug2(9, "Read all at once:\n@socket_data\n");
	&CCDebug2(9, "Finished reading response.\n");

    if ($OS eq 'UNIX') { 
		# turn off the alarm and reset the default signal handler
		alarm(0);
		$SIG{'ALRM'} = 'DEFAULT';
    }

	return ($E_NoErr, $socket_data);
}

#
# This routine is called when a SIGALRM is received.
# That means that we have timed-out waiting for a 
# response from the payment server.
# 
sub Timeout {

    $TimedOut = 1;
}


##########################################################
#
# Error reporting routines
#

sub FailureResponse 
{
	my ($status, $response) = @_;
	my($message);

	my(%result);

    $result{'MStatus'} = 'failure-hard';
    $result{'MErrLoc'} = 'CCMckDirectLib3_2';

	if ($status == $E_SocketFailed) {
    	$result{'MErrMsg'} = 'Could not open socket to connect to Merchant Payment Server.';
	}
	elsif ($status == $E_ConnectFailed) {
    	$result{'MErrMsg'} = <<'END';
Could not connect socket to the Merchant Payment Server. 
It may not be running or your configuration may be incorrect.
END
	}
	elsif ($status == $E_SyswriteFailed) {
    	$result{'MErrMsg'} = 'Could not write to the socket.';
	}
	elsif ($status == $E_TimedOut) {
    	$result{'MErrMsg'} = "No response in $timeout seconds from the Merchant Payment Server.";
	}
	elsif ($status == $E_GoAway) {
    	$result{'MErrLoc'} = 'MPMT';
    	$result{'MErrMsg'} = 'Misconfigured server/CGI-script secret.';
	}
	elsif ($status == $E_No_SSL) {
    	$result{'MErrMsg'} = "The MCK Direct Connection Protocol does not support SSL.";
	}
	elsif ($status == $E_BadMethod) {
    	$result{'MErrMsg'} = "Bad Request Method";
	}
	elsif ($status == $E_GoAway) {
    	$result{'MErrLoc'} = 'MPMT';
    	$result{'MErrMsg'} = 'Misconfigured server/CGI-script secret.';
	}
	elsif ($status == $E_Not_HTTP) {
    	$result{'MErrMsg'} = <<'END';
Invalid response received from HTTP server: 
The HOST and PORT in your configuration does not seem to *BE* an HTTP server!
END
	}
	elsif ($status == $E_Failed_HTTP) {
		$message = 	"The HTTP server has not processed your request.\n";
		$message .= "Here is the response status that it returned: \n";
		$message .= "  $response \n";

    	$result{'MErrMsg'} = $message;
	}
	elsif ($status == $E_NoConfig) {
    	$result{'MErrLoc'} = 'MCK Configuration';
		$result{'MErrMsg'} = "$response\n";
	}
	elsif ($status == $E_EncryptFailed) {
		$result{'MErrMsg'} = "Encryption failed\n";
	}
	elsif ($status == $E_DecryptFailed) {
		$result{'MErrMsg'} = "Decryption failed\n";
	}
	elsif ($status == $E_SignatureFailed) {
		$result{'MErrMsg'} = "Signature failed\n";
	}

	return %result;
}

1;
