# --*-PERL-*--
#####################################################################
#
# Copyright 1996, CyberCash Inc. All rights reserved.
# Written by Brian Boesch and Judy Grass
# Modified by Gene Gotimer
#
# These routines are intended to be used exclusively with the 
# CyberCash secure payment system.
#
# RCSID: $Id: CCMckLib3_2.pm,v 1.23 1999/03/10 22:29:56 grass Exp $
#
# Do not redistribute without written permission from CyberCash Inc.
######################################################################

package CCMckLib3_2;

require 5.003;

require Exporter;
@ISA = (Exporter);
@EXPORT = qw();
@EXPORT_OK = qw($MCKversion urlencode genURLencodedForm URLdecodeForm 
	%Config $defaultCurrency InitConfig InitDebug CCError CCDebug CCDebug2 
	CCSetTimeout
	Trim Clean  ValidateOrderId
	GetQuery 
	GetHash BuildSignature BuildPaymentFormArgs
	LoadBlockFromQuery
	BuildBlock BuildAppletParams
	PrintTemplate StringTemplate GeneratePaymentPage Browser);

use Time::Local;


use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr $E_ERROR
		$E_Bad_Timeout
		$E_No_Config 
		$E_No_File
		$E_No_Template
		$E_Null_Result
	);


#Set the CyberCash CashRegister version number
$MCKversion = '3.2.0.6';

#
#  Get appropriate TMP directory per OS
#
$OS = 'UNIX';

$TMPDIR = "/tmp/";
$FILESEP = '/';
 
if ($OS eq 'NT') {
    $TMPDIR = $ENV{'TEMP'} || "c:\\temp";
 
    unless ($TMPDIR =~ /.*\\$/) {
        $TMPDIR = "$TMPDIR\\";
    }
 
    $FILESEP = '\\';
}

$computehash = "/home/httpd/test-store/cgi-bin/mck-cgi/computeMD5hash";

#
# Debugging: Modify the variables below to enable debugging to a log file
#
# Be aware that performance is greatly decreased with debugging turned on
#

# Debugging is on if $debugging is not 0
# Set to 1 for API level debugging: only what comes into sendmserver
#    and GetQuery and what goes out of sendmserver and GetQuery is
#    logged
# Set to 2 for debugging every step of the way
# You can set it higher than two and catch other levels of debugging.
# this routine uses levels 0, 1 and 2.
#  At level 0. error messages will still go to the log if
# error_output_flag is 1.


$debugging = 2;

# Debug messages will be written to this file
$debuglog = $TMPDIR . "CCMck.log";

#Do you want errors logged? default is yes.  (set to 0 for no)
$error_output_flag = 1;

# Leave this variable alone, it is used to control the level of 
# indent for debugging messages
$indent = '';

# This is used as a global variable that holds configuration
# info:  requires a call to InitConfig to initialize
%Config = ();


# the default currency may be overriden with the configuration data
# Change it there if 'usd' is not acceptable.
# use the right ISO 4217 code for your currency
# Contact CyberCash for the current list of accepted currencies

$defaultCurrency = "usd";  # our default

sub InitConfig
{
	my($ConfigFile) = @_;

	unless(open(CONFIG, "<$ConfigFile")) {
		return ($E_No_Config);
	}
		
	# Load prms
	 while (<CONFIG>) {

		if (/^\s*\#/) {
			# skip comments
		}
		elsif (/^\s*$/) {
			# skip blank lines too
		}
		elsif ( /^\s*([\w]+)\s*=(.+)/ ) {
			# this is for real...
			$field = $1;
			$val  = $2;
			
			$val = &Trim($val);

			$field =~ tr/a-z/A-Z/;

			$Config{$field} = $val;

		}
		# anything else isn't useable

	}
	
	&InitDebug($Config{"DEBUG_FILE"}, $Config{"DEBUG_LEVEL"});
			
	close(CONFIG);

		# override the default if a currency is defined

	if (defined $Config{"DEFAULT_CURRENCY"} ) {
		$defaultCurrency = $CONFIG{"DEFAULT_CURRENCY"};
	}
	
	return ($E_NoErr);  # OK.
}

#  This lets you reset the timeout in the configuration  
#   Timout is the number of seconds the Direct Connection socket will wait
#   for a response from the CyberCash Cash Register before giving it up
#   as a lost cause.
#  
#  Your configuration file may specify a default timeout.
# 
#  If your configuration does not specify a timeout,
#  the default value will be set to 90 seconds.
# 
#  The only time this is inadequate is when waiting for the status of a
#  big batch. The processor may take more than 90 seconds to provide the
#  status. If the batch size is more than 30, it is saver to increase
#  this time out.
# 
#  INPUT:
#    pConfig      - Pointer to your active configuration record
#    timeout      - new time out value to be assigned to the configuration.
# 
#  RETURN:
#    previous timeout value, -1 if error. (allows you to backup..)

sub CCSetTimeout 
{
	my ($timeout) = @_;
	my ($oldTimeout);
	
	if ($timeout <= 0) {
		return $E_Bad_Timeout;
	}

	$oldTimeout = $Config{"TIMEOUT"};
	$Config{"TIMEOUT"} = $timeout;
	return $oldTimeout;
}

###################
# InitDebug
#
# Set up a file and a level... 
##################

sub InitDebug {
	my ($filenm, $level) = @_;

	if ($filenm) {
		$debuglog = $filenm;
	}

	if ($level) {
		$debugging = $level;
	}
}

###############
#  CCError
# 
#  PURPOSE:
# 	 To log error messages to standard error & error log file.
# 
#  INPUT:
# 	 message -- the string to dump
# 
##############

sub CCError{
	my($message) = @_;

	# check for error output flag
	if ($error_output_flag == 0) {
		return;
	}

	# Write to stderr (if you can.. no guarentee here)
	# write to debug log
	# even if debugging is off...
	&CCDebug2(0, "\nError: $message\n");
}

################
#  CCDebug
#  
#  PURPOSE:
#   Write to debug log.
# 
#  INPUT:
#   fmt			- Format that is the same as the one supported by printf().
#   ...			- parameters matching the format just like printf().
#
##################
  
sub CCDebug {
	my($message) = @_;

    if ($debugging >= 1) {
        if(!open(DEBUG, ">>$debuglog")) {
			return;   #Don't die because of this!
		}
        print DEBUG $message;
        close(DEBUG);
    }
	
}


###################
#  CCDebug2
#  
#  PURPOSE:
#   Write to debug log for a given level. If the _cc_debug_flag is numerically
#   lower than the level presented, the message will not be log.
# 
#  INPUT:
#   level		- Level of this error message.
#   fmt			- Format that is the same as the one supported by printf().
#   ...			- parameters matching the format just like printf().
#  
###############

sub CCDebug2 {
	my ($level, $message) = @_;

    if (!$level || ($debugging >= $level)) {
        if(!open(DEBUG, ">>$debuglog")) {
			return;  #Don't die because of this...
		}
        print DEBUG $message;
        close(DEBUG);
    }
}

#########################
# Trim:  remove leading and trailing whitespace.
#########################

sub Trim {
	my ($string) = @_;

	$string =~ s/^\s+//;
	$string =~ s/\s+$//;

	return $string;
}

##########################################################
# Clean
#
# Strip any unwanted characters from a given string,
# in this case, anything but a-z, A-Z, 0-9, plus, minus,
# space, tab, slash, at, period, percent, or backslash
# Most importantly, semicolons and dollar signs are removed
# so they won't get passed to the command line.
#
sub Clean {

    my($string) = @_;

# check that all characters are legal, 
# if not, delete illegals
    $string =~ tr/a-zA-Z0-9_\-+ \t\/\@\.%\\//cd;

    return $string;
}

#################################
# ValidateOrderId
#
# This subroutine tests an order ID for correct format
# 
# Don't mess with this... it tests for the format expected at the Cash Register
#
#################################

sub ValidateOrderId {
    my($orderID) = @_;
	my($len);

    # You can use any scheme to generate order ID you want, 
    # so long as:
    #   1) the order ID is unique,
    #   2) it is 32 characters or less, and
    #   3) the following characters are not used 
    #         : < > = + @ " % = & $ *

	$len = length($orderID);
	if (($len > 32) || ($len <=0)){
		return 0; #wrong size
	}

    if ($orderID =~ /[:<>=+@"%&\$\*]/) {
		#if any of these occur here, it fails!
		return 0;
	}

    return 1;
}

##############
#  urlencode
#
#  urlencode a single string
###############

sub urlencode {

    my ($string) = @_;

    # only alphanumerics and underscore (\w), dash, period, and star
    # can go through UNencoded
	# note: \w is subject to locale... the explicit list here is not
    $string =~ s/([^ a-zA-Z0-9_\-.*])/uc sprintf("%%%2.2X",ord($1))/ge; 

    # spaces get converted to plus signs (+)
    # plus signs were already converted to %2B (ascii 43)
    $string =~ s/ /+/g;

    return $string;
}


##########################################################
# genURLencodedForm
#
# URLencode an entire html form  message and place into "cybermessage"
# genURLencodedForm takes a list elements 0,2,4,... are fieldnames
#                        elements 1,3,5,... are values
#
#######################

sub genURLencodedForm {
    my(@msglist) = @_;
    my($msg) = '';
    my($name,$value);
	my($blind);   #do we need to blind the value? Yes for any card number or
				  # Account number

    &CCDebug2(5, "\n${indent}== Entering genURLencodedForm ==\n");
    $indent .= '  ';

    # step through the list
	while (@msglist > 0) 
	{

		$blind = 0;
		# grab the next name-value pair
		$name = shift(@msglist);
		$value = shift(@msglist);

	
		if (($name =~ /^cpi\./)
			|| ($name =~ /^CPI/)
			|| ($name =~ /card-/)
			|| ($name =~ /check-/)) 
		{
			$blind = 1;
			&CCDebug2(5, "${indent}Raw (Encoded) name-value pair: $name -> ****\n");
		}
		else {
			&CCDebug2(5, "${indent}Raw (Encoded) name-value pair: $name -> $value\n");
		}

		$name = &urlencode($name);
		$value = &urlencode($value);


		if ($blind == 1) {
			&CCDebug2( 5, " ($name -> ****)\n");
		}
		else {
			&CCDebug2( 5, " ($name -> $value)\n");
		}

		# add the now-encoded name-value pair to the message
		$msg .= "$name=$value";

		# add an ampersand (as a name-value pair delimiter) unless
		# we have reached the end
		$msg .= '&' unless (@msglist <= 0);
   	}

	# everything is url encoded, so send the message back
	$indent =~ s/  //;
	&CCDebug2 (5, "${indent}== Exiting genURLencodedForm ==\n\n");

    return $msg;
}


##########################################################
# URLdecodeForm
#
# URL decode an HTML Form encoded message
#######################

sub URLdecodeForm {
	my($encoded) = @_;
	my(@pairs) = split(/&/,$encoded);
	my(%pairs,$name,$value);

	my($blind);  # Doe we need to blind value? Yes for account numbers...

	&CCDebug2 (5, "\n${indent}== Entering URLdecodeForm ==\n");

	$indent .= '  ';

	foreach (@pairs) {

		$blind = 0;

		# Split into name and value
		($name,$value) = split(/=/,$_,2);

		if (($name =~ /^cpi\./)
			|| ($name =~ /^CPI/)
			|| ($name =~ /card-/)
			|| ($name =~ /check-/)) 
		{
			$blind = 1;
			&CCDebug2( 5, "${indent}Raw name-value pair: $name=****\n");
		}
		else {
			&CCDebug2( 5, "${indent}Raw name-value pair: $_\n");
		}

		# Convert plus signs to spaces, and %XX from hex numbers to alphanumeric

		$name =~ s/\+/ /g;
		$name =~ s/%([a-fA-F0-9]{2})/pack("c",hex($1))/ge;
		$value =~ s/\+/ /g;
		$value =~ s/%([a-fA-F0-9]{2})/pack("c",hex($1))/ge;

		# If the name has already been seen, just tack the value on the end
		# with a \0 delimeter      
		if ( defined($pairs{$name}) ) {
		    $pairs{$name} .= "\0$value";
		} else {
		    $pairs{$name} = $value;
		}

		if ($blind != 0) {
			&CCDebug2 (5, "${indent}Decoded name-value pair: $name -> ****\n\n");
		}
		else {
			&CCDebug2 (5, "${indent}Decoded name-value pair: $name -> $pairs{$name}\n\n");
		}
	}

	$indent =~ s/  //;
	&CCDebug2 (5, "${indent}== Exiting URLdecodeForm ==\n\n");

	return %pairs;
}


##########################################################
#
# GetQuery
#
# Get query parameters from an HTTP message (GET or Post HTML form)
#####################################################

sub GetQuery {
	my($query,%pairs) = '';
	my($name);

	&CCDebug2(1,"\n${indent}== Entering GetQuery ==\n");
	$indent .= '  ';
	&CCDebug2(1, "${indent}$ENV{'REQUEST_METHOD'} request:\n");

	# Read in text
	if ($ENV{'REQUEST_METHOD'} =~ /GET/i) {
		$query = $ENV{'QUERY_STRING'};

		# We do not log raw query info because
		# it may contain sensitive data:  like credit card or checking
		# account numbers.  If you need to see that kind of thing,
		# uncomment this line and set the debug level as needed

		# &CCDebug2( 6, "$query\n");

	} elsif ($ENV{'REQUEST_METHOD'} =~ /POST/i) {
		read(STDIN,$query,$ENV{'CONTENT_LENGTH'}) if ($ENV{'CONTENT_LENGTH'} > 0);

		&CCDebug2 (6, "${indent}Content-Length = $ENV{'CONTENT_LENGTH'}\n");

		# We do not log raw query info because
		# it may contain sensitive data:  like credit card or checking
		# account numbers.  If you need to see that kind of thing,
		# uncomment this line and set the debug level as needed

		# &CCDebug2( 6, "$query\n");
	}

	# Decode the results
	%pairs = &URLdecodeForm($query);

	if ($debugging) {
		&CCDebug2 (1, "${indent}## Decoded Name-Value pairs ##\n" );
		foreach (keys %pairs) {
			$name = $_;
			if (($name =~ /^cpi\./)
				|| ($name =~ /^CPI/)
				|| ($name =~ /card-/)
				|| ($name =~ /check-/)) 
			{
				# blind anything that looks like an account number
				&CCDebug2 (1,"${indent}$name = [[****]]\n");
			}
			else {
				&CCDebug2 (1, "${indent}$name = [[$pairs{$name}]]\n");
			}
		}
		$indent =~ s/  //;
		&CCDebug2 (1, "\n\n${indent}== Exiting GetQuery ==\n\n\n");
	}

	return %pairs;
}


################
# LoadBlockFromQuery
# 
# This is MCK specific
#
# Get any fields in query that have the specified prefix
# and add them to the PaymentNVlist block
################

sub LoadBlockFromQuery {
	my($blockname, $queryRef, $payNVListRef) = @_;
	my(%query, %paymentNVList); 
	my($key);

	%query = %$queryRef;
    %paymentNVList = %$payNVListRef;

	if ($blockname eq "*") {
			# the block is ALL of it
		foreach $key (keys (%query)) {
			$paymentNVList{$key} = $query{$key};
		}
	}
	elsif ($blockname eq "") {

		# unprefixed elements 
        # when they add "attributes" to the mix, this will need 
        # to be revisited                                       

		foreach $key (keys (%query)) {
			if (index($key,".") <= 0) {   # treat ".nnn" as unprefixed label
				$paymentNVList{$key} = $query{$key};
			}
		}
	}
	else {
		$prefix = "$blockname.";
		foreach $key (keys (%query)){
			if (index($key, $prefix) == 0) {  # found it!
				$paymentNVList{$key} = $query{$key};
			}
		}
	}

	return %paymentNVList;
}


################
# GetHash
#
# Given a string and a hash secret, generate a Base-64 encoded MD5 hash string 
#################

sub GetHash {
    my($text,$secret) = @_;
    my($signThis) = "${secret}${text}${secret}";

    # can't use command line to pass text on NT, so
    # write the message to a temporary file
	#  Do this in Unix too, since exec on anything from user
	#  put on the command line is pretty suicidal!

    my($tempfile) = "hash$$";
    $tempfile =~ tr/A-Za-z0-9_//cd;
	
    $tempfile = $TMPDIR . "$tempfile.tmp";

	# Input may be sensitive, so pipe it in...
    open(HASH,"| $computehash >$tempfile");

    &CCDebug2 (6, "${indent}##### in GetHash ######\n");

		# We do not log the hashing input because
		# it may contain sensitive data:  like credit card or checking
		# account numbers.  If you need to see the input,
		# uncomment this line and set the debug level as needed

    #&CCDebug2 (6, "${indent} signing:\n$signThis\n");
		
    print HASH $signThis;
    print HASH "\r\n";   # force a flush
    close(HASH);

    # get the base64-encoded MD5 hash
    open(TEMP,"< $tempfile");

		
    $hash = <TEMP>;
    chomp($hash);
    close(TEMP);

    # strip leading and trailing whitespace
    $hash = &Trim($hash);


    # clean up the temporary file
    unlink "$tempfile";

    &CCDebug2 ( 6, "Result is $hash\n");
    return($hash);
}

################
# BuildBlock
# 
# This is MCK specific
#
# Build a name value pair list  from a list of pairs
# by extracting all items with "blockname" prefix...
# if blockname = "*",  generate a signature based on the entire block
# if the blockname is "" (null), generate a signature on fields 
# with names that do *not* have a prefix 
#  (e.g. do not contain the dot char)
#  Note: '.' is a reserved character
################

sub BuildBlock {
	my($blockname, %fields) = @_;
	my(%block, %key);

	if ($blockname eq "*") {
			# the block is ALL of it
		%block = %fields;
	}
	elsif ($blockname eq "") {

		# unprefixed elements 
        # when they add "attributes" to the mix, this will need 
        # to be revisited                                       

		foreach $key (keys (%fields)) {
			if (index($key,".") <= 0) {   # treat ".nnn" as unprefixed label
				$block{$key} = $fields{$key};
			}
		}
	}
	else {
		$prefix = "$blockname.";
		foreach $key (keys (%fields)){
			if (index($key, $prefix) == 0) {  # found it!
				$block{$key} = $fields{$key};
			}
		}
	}

	return %block;
}

######################
# BuildSignature
#
# This is MCK specific
#
# Extract a block from an input name-value pair list 
# and generate an MD5hash signature for it:
#
#    * sort fields in a block by alphabetical order on the field key.
#    * Concatenate the values in that order
#    * add secret to front and back
#    * strip white space
#    * apply MD5hash.
#
#  return the signature
#################

sub BuildSignature {
	my($blockname, $secret, %fields) = @_;
	my ($fieldName, $hashString, $sign);

	my ($prefix, %hashBlock);

	%hashBlock = &BuildBlock($blockname, %fields);

	$hashString = "";

	# " When we translate a message block to a string, we don't want 
	# to lose the tags or the delimiters between names and values.
	# To do this, we serialize a block as follows:
   	# 	<name1><space><value1><CR>...<CR><nameN><space><valueN> "
	#  (Gospodi, Pomiluyi..).

	foreach $key (sort keys (%hashBlock)) {

		if ($hashString ne "") {
			$hashString .= "\r";
		}
		
		$hashString .= "$key ";	
		$hashString .= $hashBlock{$key};
	}

	$sign = &GetHash($hashString, $secret);

	return $sign;
}


######################
# BuildPaymentFormArgs
# 
# This is MCK specific
#
# Build the input lines for an HTML form 
# that will submit  a payment request to the Cash Register
#
###################

sub BuildPaymentFormArgs 
{
	my($MOref, $MFref) = @_;
	my(%MO, %MF);
	my($body, $tbody);

	%MO = %$MOref;
	%MF = %$MFref;

	$body = "";
	$tbody = genURLencodedForm(%MO);

	$body .=  "<input type=hidden name=\"MO\" value=\"$tbody\">\r\n";
	
	if ($#MF != 0) {   #ignore if empty
		$tbody = genURLencodedForm(%MF);
		$body .=  "<input type=hidden name=\"MF\" value=\"$tbody\">\r\n";
	}

	return $body;
}


######################
# BuildAppletParams
#
# This is MCK specific
#
# Build the parameter lines for an applet
# that will submit a payment request to the Cash Register
#
###################

sub BuildAppletParams
{
        my($MOref, $MFref) = @_;
        my(%MO, %MF);
        my($body, $tbody);

        %MO = %$MOref;
        %MF = %$MFref;

        $body = "";
        $tbody = genURLencodedForm(%MO);
        $body .=  "<param name=\"MO\" value=\"$tbody\">\r\n";

        $tbody = genURLencodedForm(%MF);
        $body .=  "<param name=\"MF\" value=\"$tbody\">\r\n";

        return $body;
}


##########################
# GeneratePaymentPage
#
# Given this input:
#   $paymentTemplate -- HTML template that will have the payment request
#                    embedded in it
#   $tempNVListref   -- a pointer to a list of HTML template parameters
#   $argNVListref    -- a pointer to a list of Payment Parameters
#
# Generate the PaymentPage that will be displayed to the consumer
# and launch the CashRegister payment request
########################

sub GeneratePaymentPage
{
	my ($paymentTemplate, $tempNVListref, $argNVListref) = @_;
	my (%templateArgs, %paymentArgs, $HTML_header, $token);
	my (%MO, %MF);
	
        %templateArgs = %$tempNVListref;
        %paymentArgs = %$argNVListref;

	$HTML_header = "Content-type: text/html\r\n\r\n";
	
	#the actual message is done by reading and expanding a payment template...

	if (!open (PAYMENT, "<$paymentTemplate")) {
		return $E_No_Template;   # need to define some codes..
	}

	# Build blocks
        %MO = &BuildBlock("mo", %paymentArgs);
        %MF = &BuildBlock("mf", %paymentArgs);

	#Dumb as dirt templating....

	print $HTML_header;

	while(<PAYMENT>) {
		$line = "";
		$inline = $_;
			
			# Allow multipletokens on a line
			# token replacement is *NOT* recursive
			# Consume $inline, output goes to $line

		while ($inline =~ /#([\w]+)#/) {
			$token = $1;
			if (defined($templateArgs{$token})) {
					# substitute the value for the token 
				$line .= "$`" . $templateArgs{$token};
				$inline = "$'";
			}
			elsif ($token eq "MANUAL_ARGS") {
				$line .= "$`" . &BuildPaymentFormArgs(\%MO,\%MF);
				$inline = "$'";
			}
			else {
				
				# no defined value for this token,
				# so do not replace it

				$line .= "$`#$token#";
				$inline = "$'";
			}
		}
		$line .= $inline;

		print $line;
	}

	close(PAYMENT);

	return $E_NoErr;    # all is well
}

##########################################################
# Browser
#
# Determine the browser type that the consumer is using
# from data in the HTTP request headers (HTTP_USER_AGENT)
#
################################

sub Browser {
    my(%browser) = ();
    my($platform,$type,$version);

    $_ = $ENV{'HTTP_USER_AGENT'};

    # get the platform
    if (/win(dows)?\s*(32|95|nt)/i) {
        $platform = 'win32';
    } elsif (/win/i) {
        $platform = 'win';
    } elsif (/mac/i) {
        $platform = 'mac';
    } else {
        $platform = 'other';
    }

    # get the browser type and version
    # First, check explicitly for MSIE
    if (/MSIE/) {
        $type = 'MSIE';
        ($version) = /MSIE\s+(.+?);/;
    } else {
        ($type,$version) = /^(.*?)\/\s*(.+?)\s/;
    }

    # put it into the assoc array
    %browser = ( 'platform' => $platform,
                 'type'     => $type,
                 'version'  => $version );

    return(%browser);
}

####################################
#  PrintTemplate
#  
#  Inputs:
#    $template  -- full path to a template that must be expanded and output
#    %tokens  -- list of token names and values to substitute in the template
#
#    Read the template file, substitute values for tokens in the template
#    and dump the results to STDOUT.
#
#  Return: status.  0 is OK, anything else indicates a failure.
#
#####################

sub PrintTemplate
{
	my($template,  %tokens) = @_;
	my ($val, $tokenName, $valName);
	my ($tempFile, $pos);
	my ($errstr);

	# Then, read template and print it, substituting valid tokens
	unless (open(TEMPLATE,"<$template")) {
		$pos = rindex($template, $FILESEP);
		if ($pos <= 0) {
			$tempFile = $template;
		}
		else {
			$tempFile = substr($template, $pos+1);
		}
		
		$errstr = &MCKGetErrorMessage($E_No_Template);

    	&CCError("$0: $errstr ($template): $!");
    	return $E_No_Template;
	}

	print "Content-type: text/html\r\n\r\n";
	while (<TEMPLATE>) {
		while (/\#([\w-.:;]+)\#/) {
			$tokenName = $1;

			if ($tokenName =~ /^url-/) {
				# need to url-encode the value before substitution
				# strip the prefix

				$valName = substr($tokenName, 4);
				if (defined ($tokens{$valName})) {
					$val = &urlencode($tokens{$valName});
				}
				else {
					$val = " ";
				}
			}
			elsif (defined ($tokens{$tokenName})) {
				$val = $tokens{$tokenName};
			}
			else {
				$val = " ";
			}

			s/\#$tokenName\#/$val/;
    	}
    	print;
	}
	close(TEMPLATE);

	return $E_NoErr;
}

#######################
# StringTemplate
#
#  Inputs:
#    $template  -- full path to a template that must be expanded and output
#    %tokens  -- list of token names and values to substitute in the template
#
# read the template and build a string from it, replacing
# tokens as you go...
#
#  Return $status, and resulting string
#    if status = $E_NoErr (0), else this failed
#
#########################

sub StringTemplate
{
	my($template,  %tokens) = @_;
	my ($val, $tokenName, $valName);
	my ($tempFile, $pos);
	my($errstr);

	my($outStr);

	$outStr = "";

		# maybe no note is needed?
	if (!$template) {
		       # maybe this is OK?
        return ($E_Null_Result, $outStr);
	}
	

	# Then, read template and build a string  from it, substituting valid tokens
	unless (open(TEMPLATE,"<$template")) {
		$pos = rindex($template, $FILESEP);
		if ($pos <= 0) {
			$tempFile = $template;
		}
		else {
			$tempFile = substr($template, $pos+1);
		}
		
		$errstr = &MCKGetErrorMessage($E_No_Template);

    	&CCError("$0: $errstr ($template): $! ");
    	return ($E_No_Template, "");
	}

	while (<TEMPLATE>) {
		while (/\#([\w-.:;]+)\#/) {
			$tokenName = $1;

			if ($tokenName =~ /^url-/) {
				# need to url-encode the value before substitution
				# strip the prefix

				$valName = substr($tokenName, 4);
				if (defined ($tokens{$valName})) {
					$val = &urlencode($tokens{$valName});
				}
				else {
					$val = " ";
				}
			}
			elsif (defined ($tokens{$tokenName})) {
				$val = $tokens{$tokenName};
			}
			else {
				$val = " ";
			}

			s/\#$tokenName\#/$val/;
    	}
    	$outStr .= $_;
	}
	close(TEMPLATE);

	return ($E_NoErr, $outStr);
}

1;
