# --*-PERL-*--
#####################################################################
#
# Copyright 1996, CyberCash Inc. All rights reserved.
# Written by Judy Grass
#
# These routines are intended to be used exclusively with the 
# CyberCash secure payment system.
#
# RCSID: $Id: CCMerchantTest.pm,v 1.16 1998/12/07 14:46:20 grass Exp $
#
# This contains merchant configuration parameters
#  And common merchant customizable subroutines used by all of the MCK
#  Payment scripts.
#
# Do not redistribute without written permission from CyberCash Inc.
######################################################################

package CCMerchantTest;

require 5.003;

require Exporter;
@ISA = (Exporter);
@EXPORT = qw();
@EXPORT_OK = qw( SaveOrderInfo GetOrderInfo CheckOrderId 
	LogNotification ExtractNotification
	TemplateFailure  $progdir $ConfigFile);

use CCMckLib3_2 qw(%Config $defaultCurrency InitDebug Clean Trim ValidateOrderId
	urlencode URLdecodeForm
	CCError CCDebug CCDebug2);

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr
		$E_No_OrderLog
		$E_No_NotifLog
		$E_Order_NotFound
	);

$progdir = &getRelPath();

# this is hard coded here... gotta start somewhere!

$OS = 'UNIX';

$ConfigFile =  $progdir . "/conf/merchant_conf";
 
################################
# TemplateFailure 
#
# This is here for total melt-down of templating.
# Otherwise, templates will be expanded for messaging.
#
# Please customize the text here if you wish.
#
##########################################

sub TemplateFailure 
{
	my($message) = @_;
	print "Content-type: text/html\r\n\r\n";
	
	print <<"END";
<html>

<head>
<title>Temporary Difficulties</title>
</head>

<body>
<h1>Temporary Difficulties</h1>

<p>CyberCash MCK Test Merchant is experiencing 
temporary difficulties.</p>

$message

<p>Please contact CyberCash MCK Test Merchant
customer service staff at merchant-support\@cybercash.com, 
or try again later.</p>

<p>Sorry for the inconvenience.</p>

</body>
</html>
END

}

#########################################################
# 
#  SaveOrderInfo
#
# This routine is used by the testdriver CGI
#
#  It stores information associated with an order into a flat file
#  This includes:
#  $orderId   -- the order ID, which is used as a key for lookup
#  $price     -- How much?  Format:  usd 99.99
#  $orderValidTill -- (optional) expiration date in any format you desire
#  $payload   -- (optional) file being sold (or whatever) for soft goods sale.
#  $orderDescr -- (optional) textual description goods sold
#  $customerId -- (optional) Who is the customer?
#	
# This file is used two ways:
#   1) For looking up the order information so that it can be included in
#      the payment message sent to the CyberCash Cash Register.
#   2) For verifying that a notification that arrives at the merchant
#      contains an order-id that the merchant generated and can recognize.
#      If it does not, then there is a problem (fraud or malfunction?)
#
# Here:  I'm using a flat file.  
#   The key is the orderId, 
#   I store all of tha above parameters 
#   *AND* I add a timestamp.....
#  This will make it possible for me to delete orderId's from this
#  database at some point when they are too old to be of interest.
#
# This approach will not scale for a large and busy store.  
# You will need to write your own code to store Order Information when
# a sale is made and to retrieve it when the payment messages are built
# in the payment CGI scripts.
#
# WARNING: I am using '&' as a record separator...
# do not use this character in orderIds, prices(?), orderValidTill, payloads
# or customerIds unless you make changes to accommodate this!
#
####################

$ORDER_RSEP = '&';

sub SaveOrderInfo {
    my($orderId, $price, $orderValidTill, $payload, 
		$orderDescr,$customerId) = @_;

    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time());

    my($timestamp);

	# year is tm.tm_year:  years since 1900
    $timestamp = sprintf("%04d%02d%02d%02d%02d",
    		     $year + 1900,$mon+1,$mday,$hour,$min);

		#orderDescr is free text... urlencode it to be safe..
		# this will hide newlines, whitespace and renegade ampersands

	$codedOrderDescr = &urlencode($orderDescr);

	#&CCDebug("coded is :\n$codedOrderDescr\n");
    # should add locking here...
    
    unless(open (ORDERDB, ">>".$Config{"ORDER_DB"})) {
    	return $E_No_OrderLog;  # DB failure
    }

    print ORDERDB $orderId 
		. $ORDER_RSEP . $timestamp 
		. $ORDER_RSEP . $price 
		. $ORDER_RSEP . $orderValidTill
		. $ORDER_RSEP . $payload 
		. $ORDER_RSEP . $customerId
		. $ORDER_RSEP . $codedOrderDescr
		. "\n";
    close(ORDERDB);
  
    return $E_NoErr;  # a-ok
}

##############################
# sub GetOrderInfo
#
# When the order was generated, the OrderId was saved along with
# price data and other info needed to generate a payment request..
# (see SaveOrderInfo)
#
# Now, check to see if the OrderId on the notification is an OrderId 
# that you recognize and extract all of the relevant info from it.
#  
# You will need to replace this function with something more
# scalable and better integrated with your storefront.
#
# Note:  The lookup in this sample code is done using a Perl
#   regular expression.  If your OrderId contains characters of strings
#   that look like Perl regular expression operators (., $, ^, {, }, etc.)
#   or Perl regular expression tokens (\n, \w, \D, etc.) 
#   you may get unexpected results.  
#
############################

sub GetOrderInfo 
{
    my($orderId) = @_;
	my($found) = 0;
	my($line) = "";
    my($oid, $timestamp, $price, $orderValidTill, $payload, 
		$customerId, $orderDescr);
	my($rest);

	# should add locking here...

	unless(open (ORDERDB, "<".$Config{"ORDER_DB"})) {
		return $E_No_OrderLog;  # DB failure
	}
		
	# ** sigh! ** linear search...
	SEARCH:
	while (<ORDERDB>) {
		if (/^$orderId$ORDER_RSEP/) {
			$found = 1;
			$line = $_;  # here is the data
			
			last SEARCH;
		}
	}

	close(ORDERDB);

	if ($found) {
    	($oid, $timestamp, $price, $orderValidTill, $payload, 
		$customerId, $orderDescr, $rest) = split($ORDER_RSEP, $line);

			# trim leading and trailing whitespace...
			# shouldn't be any because of how data was entered,
			# but just to be safe... (someone may have hand edited the file!)

		$oid = &Trim($oid);
		$timestamp = &Trim($timestamp);
		$price = &Trim($price);
		$orderValidTill = &Trim($orderValidTill);
		$payload = &Trim($payload);
		$customerId = &Trim($customerId);

				# urldecode the order description text
				# leading and trailing whitespace is OK here
				# since we don't look at this

		$orderDescr =~ s/\+/ /g;
        $orderDescr =~ s/%([a-fA-F0-9]{2})/pack("c",hex($1))/ge;

	}
  
    return ($found, $oid, $timestamp, $price, $orderValidTill, $payload,
        $customerId, $orderDescr); 
}

##############################
# sub CheckOrderId
#
# When the order was generated, the OrderId was saved.
# Now, check to see if the OrderId on the notification is an OrderId 
# that you recognize.
#
#  See comments in SaveOrderInfo to see how the test script implements
#  the Order Log.
#  
# I'm using a flat file, but this will not scale to a large, complex
#  storefront.  You will need to replace this function to work with
#  the kind of data you store and the method you use to store that data.
#
############################

sub CheckOrderId {
    my($orderId) = @_;
	my($found) = 0;

	# should add locking here...

	unless(open (ORDERDB, "<".$Config{"ORDER_DB"})) {
		return $E_No_OrderLog;  # DB failure
	}
		
	# ** sigh! ** linear search...
	SEARCH:
	while (<ORDERDB>) {
		if (/^$orderId$ORDER_RSEP/) {
			$found = 1;
			last SEARCH;
		}
	}
			
	close(ORDERDB);
  
    return $found; 
}

###################################
#  LogNotification
#
#  Write the POP to the notification log
#  If you want to change the format or what is saved,
#  make the  changes here
#
# We are saving the POP string as it arrived...
#
#  return $status, $code as below.
#   $status is numeric code (see CCMckErrno3_2.pm)
#	$code = "000" is success
#	$code = "3xx" is a file error of some kind
#
########################################

sub LogNotification
{
	my ($POPstr) = @_;

	if(!open(LOG,">>".$Config{"NOTIFICATION_LOG"})) {
		$status = $E_No_NotifLog;
		$code = "300";
		return ($status, $code);
	}
	else {

		print LOG "$POPstr\n";
		close(LOG);

		$status = $E_NoErr;
		$code = "000";
	}
	return ($status, $code);
}

###################################
#  ExtractNotification
#
#  Input:  An order ID	
#  Output: An error code   (0 = OK, > 0  errno: file problem,
#                           -1 if record not found)
#          An error message (if applicable)
#          An associative array containg the fields of the POP record
#  
#  If you change the Notification Log format or what is saved in that log,
#  You will have to make matching changes here...
#
#  Note: The POP string is stored in URL encoded format,
#  as it arrived in the directpay and/or fulfillment cgis...  
#
########################################

sub ExtractNotification
{
	my ($orderId) = @_;
	my (%POP);
	my ($line, $oid, $log);
	my ($message) = "";
	my ($status) = 0;

	%POP=();   # initialize to empty

	$log = $Config{"NOTIFICATION_LOG"};

	if(!open(LOG,"<".$Config{"NOTIFICATION_LOG"})) {
		$status = $!;
		$message = "[$!] ". &MCKGetErrorMessage($E_No_NotifLog) . " ($log)";
		return ($status, $message, %POP);
	}
	
	while (<LOG>) {
		# look for OID in each line
		$line = $_;

		# Do this on the cheap... 
		# find pop.order-id in the string, without URL decoding
		# extract its value  from the raw string.
		# urldecode that
		# if it matches our oid, ***THEN*** we'll unpack the record.

		if ($line =~ /pop.order-id=([^&]+)/ ) 
		{
			$oid = $1;

				# urldecode
			$oid =~ s/\+/ /g;
        	$oid =~ s/%([a-fA-F0-9]{2})/pack("c",hex($1))/ge;

			if ($oid eq $orderId) 
			{
				# this is it!
				%POP = &URLdecodeForm($line);
				close (LOG);
				$message = &MCKGetErrorMessage($E_NoErr);
				return (0, $message, %POP);
			}
		}
		# if no pop.order-id, this is wierd... but ignore it

	}
	$message = &MCKGetErrorMessage($E_Order_NotFound);
	return (-1, $message, %POP);
}

sub getRelPath {
  if ( $0 !~ /^\// ) {  # $0 is not an absolute path
     $s=`pwd`;          # hope pwd on path...
     chomp $s;
     $s.="/$0";
     } else {              # $0 IS an absolute path
     $s=$0;
  } # done determining if $0 is relative or absolute
  @temp=split /\//, $s;

  @rpath=@temp;
  $scriptname = pop @rpath;
  $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

  $rundir= join "/", @rpath;
  #chdir( $rundir );

  $relpath="";
  $siteshortname="";
  ## short circuit if $s does not contain cgi-bin
  if ( $s !~ /cgi-bin/ ) {
     $siteshortname = pop @rpath;
     $relpath=".";
     @temp=();
  } # done testing for degenerate case

  while ( @temp ) {
     $u = pop @temp;

     ## skip references to the present directory
     next if ( $u eq '.' );
     next if ( length($u)==0 );        # skip null directory levels

     $relpath.="/.." if ( $relpath);
     $relpath.="." if ( ! $relpath );

     if ( $u eq "cgi-bin" ) {
        $siteshortname = $temp[$#temp];
        @temp=();
     } # note - cgi-bin is off the site root, so the next up is site root
  }
  $relpath .= "/../$siteshortname";

  return $relpath;

}

1;
