#!/usr/bin/perl

#   $Id: fulfillment.cgi,v 1.20 1999/03/15 16:07:30 grass Exp $
#
#   System     :   Merchant Connection Kit (MCK)
#
#   Author     :   CYCH Development staff
#   Description:   CGI script to accept a request for fulfillment over
#                  an SSL browser link from a customer that has completed
#                  a payment
#   Notes      :   This may be edited to integrate with the merchant's
#                   storefront
#
#                         COPYRIGHT NOTICE
#
#   The contents of this file is protected under the United States
#   copyright laws as an unpublished work, and is confidential and
#   proprietary to CyberCash, Inc.  Its use or disclosure in whole or in
#   part without the expressed written permission of CyberCash, Inc. is
#   prohibited.
#
#   (c) Copyright 1993-98 by CyberCash, Inc.  All rights reserved.
#

# fulfillment.cgi
#
#	Accept and log fulfillment info from the Merchant Connection Kit
#	generate a receipt or dispense a payload
#
#    RCSID $Id: fulfillment.cgi,v 1.20 1999/03/15 16:07:30 grass Exp $
#


# 
# Check Perl version
#
require 5.003;

# Require some libraries
#


# The following 4 lines are here only because the current directory for
# Microsoft IIS when running script on browser is always the root
# directory of the web server.
BEGIN {
     push(@INC, '/home/httpd/test-store/cgi-bin/mck-cgi');
     push(@INC, '/home/httpd/test-store/cgi-bin/mck-cgi/');
}

use CCMckLib3_2 qw(%Config InitConfig GetQuery BuildSignature 
	CCError CCDebug CCDebug2 
	URLdecodeForm PrintTemplate);

use CCMerchantTest  qw($ConfigFile TemplateFailure
	CheckOrderId LogNotification );

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr
    $E_Fail_Notif
    $E_No_Receipt
	$E_POP_Signature
	$E_MF_Signature
	$E_Order_NotFound
    );

use CCMerchantCustom  qw( SendPayload );


 
# Initialize the Global Configuration array in CCMckLib3_2
($status) = &InitConfig($ConfigFile);

if ($status != 0) {
    $errmsg = &MCKGetErrorMessage($status);
    &TemplateFailure($errmsg);

	# since logging isn't possible, call die and try to get the web server
	# to log this problem
    die("$0: $errmsg.\n");
}


&CCDebug("Entering $0\n");

$mckType = $Config{"MCK_TYPE"};

$cybercashId = $Config{"CYBERCASH_ID"};
$hashSecret = $Config{"HASH_SECRET"};
$templateDir = $Config{"TEMPLATE_DIR"};


# page to dump if there are problems with fulfillment
$problemTemplate = $templateDir . "failFulfillment.tem";
$failureTemplate = $templateDir . "customFailureResponse.tem";
$thankYouTemplate = $templateDir . "customReceipt.tem";

# get the query string
%query = &GetQuery;

$POPstr = $query{'POP'};
delete $query{'POP'};
%POP = URLdecodeForm($POPstr);
$orderId = $POP{'pop.order-id'};
$price = $POP{'pop.price'};

$MFstr  = $query{'MF'};
delete $query{'MF'};
%MF = URLdecodeForm($MFstr);

$Codestr  = $query{'CODE'};
delete $query{'CODE'};

$Messagestr  = $query{'MESSAGE'};
delete $query{'Message'};

$Resultstr  = $query{'RESULT'};
delete $query{'RESULT'};

if ($Messagestr ne "" or $Codestr ne "") {
   $templatePrms{'error-code'}= $Codestr;
   $templatePrms{'error-message'}  = $Messagestr;
   if ($orderId eq "") {
      $orderId = "Unknown Order";
   }

   if ($price eq "") {
      $price = "Unknown Price";
   }

   $templatePrms{'mo.order-id'}  = $orderId;
   $templatePrms{'mo.price'}  = $price;

   $errcode = &PrintTemplate($failureTemplate, %templatePrms);
   if ($errcode != $E_NoErr) {
		&TemplateFailure($Messagestr);
   }

   exit(1);
}

if ($Resultstr ne "") {
   print "Content-type: text/html\r\n\r\n";
   print $Resultstr;
   print "\n";
   exit(1);
}

# There may be additional items in here, if you added them
#  manually in the Payment page...
#  You know what they are, we don't...
%REMAINDER = %query;

$sign = $POP{'pop.sign'};
delete $POP{'pop.sign'};  ## don't sign the sign... and don't log the sign

$orderId = $POP{'pop.order-id'};

$tmpSign = &BuildSignature("pop", $hashSecret, %POP);

if ($tmpSign ne $sign) 
{
	#GONG!
	$errmsg = &MCKGetErrorMessage($E_POP_Signature);
	$templatePrms{'MESSAGE'} = $errmsg;
	$templatePrms{'ORDERID'} = $orderId;

	&CCError("$0: $errmsg.\n");
	$errcode = &PrintTemplate($problemTemplate, %templatePrms);
	if ($errcode != $E_NoErr) {
		&TemplateFailure ($errmsg);
	}
	exit (1);
}

# If you signed the MF, check the sign here....
# Change 0 to 1 if you want to check the signature

if (0) {
	$sign = $MF{'mf.sign'};
	delete $MF{'mf.sign'};  ## don't sign the sign... and don't log the sign
	
	$tmpSign = &BuildSignature("mf", $hashSecret, %MF);
	
	if ($tmpSign ne $sign) 
	{
		#GONG!
	
		$errmsg = &MCKGetErrorMessage($E_MF_Signature);
		$templatePrms{'MESSAGE'} = $errmsg;
		$templatePrms{'ORDERID'} = $orderId;

		&CCError("$0: $errmsg");
		$errcode = &PrintTemplate($problemTemplate, %templatePrms);
		if ($errcode != $E_NoErr) {
			&TemplateFailure($errmsg);
		}
		exit (1);
	}
}

	# Try to find this orderId in the order log.
	# If it isn't there, either the order has expired,
	# or this may be an order you never generated
	# (or someone trashed the order log)


	# if this is the merchant, check to see if the orderId is known
	# if not, this is a fulfillment center.  Skip this check.

	# If you **WANT** this test, you must define
	# MCK_CONFIG_TYPE = merchant 
    # in your merchant_conf file

if ($mckType eq "merchant") {
	$found = &CheckOrderId ($orderId);
	
	if ($found == 0){
		#GONG!
	
		$errmsg = &MCKGetErrorMessage($E_Order_NotFound);
		$templatePrms{'MESSAGE'} = $errmsg;
		$templatePrms{'ORDERID'} = $orderId;

		&CCError("$0: $errmsg ($orderId)\n");

		$errcode = &PrintTemplate($problemTemplate, %templatePrms);
		if ($errcode != $E_NoErr) {
			&TemplateFailure($errmsg);
		}
		exit (1);
	}
	elsif ($found != 1) {
	
		$errmsg = &MCKGetErrorMessage($found); 
		$templatePrms{'MESSAGE'} = $errmsg;
		$templatePrms{'ORDERID'} = $orderId;

		&CCError("$0: $errmsg\n");
		
		$errcode = &PrintTemplate($problemTemplate, %templatePrms);
		if ($errcode != $E_NoErr) {
			&TemplateFailure($errmsg);
		}
		exit (1);
	}
}

# everything checks out, so check for payload
if ($POP{'pop.payload'}) {

	$filename = $POP{'pop.payload'};
	$pStatus = &SendPayload($filename);
	
	if ($pStatus != $E_NoErr) {
		# something bad happened

		$errmsg = &MCKGetErrorMessage($pStatus) . " ($filename)";
		$templatePrms{'MESSAGE'} = $errmsg;
		$templatePrms{'ORDERID'} = $orderId;

		$errcode = &PrintTemplate($problemTemplate, %templatePrms);
		if ($errcode != $E_NoErr) {
			&TemplateFailure($errmsg);
		}
        &CCError("$0: $errmsg.\n");
		exit 1;
    }

		# now log the purchase too... If you want... 
		# If you don't want to log here, you don't have to.
	($status, $code) = &LogNotification($POPstr);

	if ($status != $E_NoErr) {
		# Uh-oh!  The customer has the goods, we are missing the log! 
		# What should we do?
	
		# Log the failure in any case... 
	    &CCError("$0: Order Id $orderId: "
	        . &MCKGetErrorMessage($E_Fail_Notif) . " - "
	        . &MCKGetErrorMessage($logStatus) ." \n"
	    	);
	    &CCError("$0: Log Entry: $POPstr\n");

		# Payload was sent... no additional stuff should go
		# to consumer

		exit(1);
	}

} else { # non-payload

    # Store fulfillment information

	# write this record to the notification file...
	# If you want to log MF and any additional prms,
	#  Modify this function...

	($status, $code) = &LogNotification($POPstr);

	if ($status != $E_NoErr) {
		# Log the failure in any case... 
		$errmsg = " Order Id $orderId: "
            . &MCKGetErrorMessage($E_Fail_Notif) . " - "
            . &MCKGetErrorMessage($logStatus);

	    &CCError("$0: $errmsg\n");
	    &CCError("$0: Log Entry: $POPstr\n");

		$templatePrms{'MESSAGE'} = $errmsg;
		$templatePrms{'ORDERID'} = $orderId;

		$errcode = &PrintTemplate($problemTemplate, %templatePrms);
		if ($errcode != $E_NoErr) {
			&TemplateFailure($errmsg);
		}
		exit(1);
	}

    # any of the fields passed in are tokens
    %tokens = ( %POP, %MF, %REMAINDER);


	#If you have a successful purchase AND it is a duplicate of an order
	# already completed, note that fact in the success templates
	#
	# There may be other actions you take on receiving a duplicate payment,
	# such as preventing a payload from being dispensed... but keep 
	# in mind that sometimes interruptions and transmission problems
	# may prevent the customer from ever having seen the original
	# successful response.
	
	if ($POP{'pop.status'} =~ /^success-duplicate/ ) {
		#add a message to the template
		$tokens{'DUPLICATE_MSG'} 
			= "<p>Please note that this order is a duplicate of one completed previously</p>\n";
	}
	else {
		$tokens{'DUPLICATE_MSG'} = " "; # put in a blank to ensure substitution
	} 


	$errcode = &PrintTemplate($thankYouTemplate, %tokens);
	if ($errcode != $E_NoErr) {
        &TemplateFailure( &MCKGetErrorMessage($E_No_Receipt) );
    }
}

&CCDebug("Exiting $0\n");

1;

