#!/usr/bin/perl

#   $Id: testdriver.cgi,v 1.13 1998/12/11 18:02:46 grass Exp $
#
#   System     :   Merchant Connection Kit (MCK)
#
#   Author     :   CYCH Development staff
#   Description:   CGI script that generates an HTML page that
#                  will invoke the payment scripts (cardin.cgi, etc.)
#                  for demonstration and testing purposes ***ONLY***
#   Notes      :   This is **NOT** meant to be an illustration of how a
#                   storefront should operate
#
#                         COPYRIGHT NOTICE
#
#   The contents of this file is protected under the United States
#   copyright laws as an unpublished work, and is confidential and
#   proprietary to CyberCash, Inc.  Its use or disclosure in whole or in
#   part without the expressed written permission of CyberCash, Inc. is
#   prohibited.
#
#   (c) Copyright 1993-98 by CyberCash, Inc.  All rights reserved.
#


###########################
#
# testdriver.cgi
#    This script generates an HTML page from a template that can be
#    used to test the payment CGI scripts.
#
#    RCSID $Id: testdriver.cgi,v 1.13 1998/12/11 18:02:46 grass Exp $
#
###########################


    # 
    # Check Perl version
    #
require 5.003;

    #
    # Require some libraries:  either install these where PERL can find them,
	#   or add code to push the directory where they live...
    #


# The following 4 lines are here only because the current directory for
# Microsoft IIS when running script on browser is always the root
# directory of the web server.
BEGIN {
     push(@INC, '/home/httpd/test-store/cgi-bin/mck-cgi');
     push(@INC, '/home/httpd/test-store/cgi-bin/mck-cgi/');
}

use CCMckLib3_2 qw(InitConfig  %Config
	CCError CCDebug CCDebug2 GetQuery 
	PrintTemplate Trim);

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr
	$E_No_Amount
	);

use CCMerchantTest qw($ConfigFile TemplateFailure SaveOrderInfo );

use CCMerchantCustom qw( GenerateOrderId);


##################
#  
#  Some data initializations....
#
##################



# Initialize the Global Configuration array in CCMckLib3_2
($status) = &InitConfig($ConfigFile);

if ($status != 0) {
	$errmsg = &MCKGetErrorMessage($status);
    &TemplateFailure($errmsg);

	# since logging isn't possible, call die and try to get the web server
	# to log this problem
    die("$0: $errmsg.\n");
}

&CCDebug("Entering $0\n");

$templateDir = $Config{"TEMPLATE_DIR"};
$problemTemplate =  $templateDir . "tempDifficulties.tem";


#############################
#  MAIN code starts here    #
#############################

	# get the query string
%query = &GetQuery;

	# Read the order-id... we need this!

$orderId = $query{"order-id"};
$orderId = &Trim($orderId);

if ($orderId eq "") {
	# Generate one...
	$orderId = &GenerateOrderId();
}

$templatePrms{"ORDERID"} = $orderId;

	
$price = $query{"price"};

$price = &Trim($price);

if ($price eq "") {
	#fail this... the driver is supposed to supply one

	$errmsg = &MCKGetErrorMessage($E_No_Amount);
	$templatePrms{'MESSAGE'} = "$errmsg\n";

	&CCError("$0: $errmsg\n");
	$errcode = &PrintTemplate($problemTemplate, %templatePrms);
    if ($errcode != $E_NoErr) {
        &TemplateFailure($errmsg);
    }
	exit 1;
}

	#Payload is optional
$payload = $query{"payload"};
$payload = &Trim($payload);


	#orderValidTill is optional
$orderValidTill = $query{"ovt"};
$orderValidTill = &Trim($orderValidTill);

	#orderDescr is optional free text (urlencode to store...)
	#Don't Trim it.

$orderDescr = $query{"order-descr"};

	#customerId is optional
$customerId = $query{"customer-id"};
$customerId = &Trim($customerId);

	#script is a radio button... name of the HTML template to load
	# and hence, which script we will test

##$script =  $templateDir . $query{"script"};
$script =  $templateDir . "directcreditsale.tem";



	# At this point, record what you know about the order so that
	# the payment script can retrieve the information
	# record the orderId as a known one... (fail if you can't record it)

$saved = &SaveOrderInfo (
	$orderId, 
	$price, 
	$orderValidTill, 
	$payload,
	$orderDescr,
	$customerId
	);

unless ($saved == $E_NoErr) {
	$errmsg = &MCKGetErrorMessage($saved);
	$templatePrms{'MESSAGE'} = $errmsg;

	&CCError("$0: $errmsg\n");
	$errcode = &PrintTemplate($problemTemplate, %templatePrms);
	if ($errcode != $E_NoErr) {
		&TemplateFailure($errmsg);
	}
	exit 1;
}


#At this point, all of the data that needs to go into the "Payment Page"
# should have been collected.
#
# PrintTemplate will expand a payment template:
#	cardsale.tem
#   sslformsale.tem
#	mswalletsale.tem
#   checksale.tem
#
# that will drive the CGI script that you wish to test....
#   
#  These parameters must be expanded (the CGI will validate these):
#   #ORDERID#
#   #TEST_PRMS# 
#

$errcode = &PrintTemplate( $script,  %templatePrms);
if ($errcode != $E_NoErr) {
	$errmsg = &MCKGetErrorMessage($errcode);
	&TemplateFailure($errmsg);
}

&CCDebug("Exiting $0\n");

exit 0;

