#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# formtools.pm
# 
# Set of form and page generation tools
#
# Version -.7.26 LCF 20000928
#    fixed nicedecimals .99 bug, again
# Version 0.7.25 JC 20000927
#    fixed nicedecimals so that .99 will go to .00 properly
# Version 0.7.24 JC 20000927
#    added quotes around select in gen_select
#    took out debug after select in gen_select
# Version 0.7.23 TG 20000922
#    Bug fixes for addVariantProduct
# Version 0.7.22 LCF 20000921
#    Rewrote nicedecimals to fix bug
# Version 0.7.21 TG 20000921
#    Check for blank $location and $codes in searchVariantEncodings
# Version 0.7.20 JC 20000913
#    oracle port
# Version 0.7.19 JC 20000807
#    bug fixes
# Version 0.7.18 TG 20000713
#    Added search routines, searchVariantEncodings, addVariantProducts,
#    getVariantCode, getVariantLocation, retrofit for language - 
#    formSearchQuery
# Version 0.7.17 TG 20000712
#    Limited number of page links in setSearchNavigation
# Version 0.7.16 TG 20000707
#    Put OR statements in () for formSearchQuery
# Version 0.7.15 TG 20000706
#    Added links to pages for search results
# Version 0.7.14 TG 20000705
#    Added objtype to formSearchQuery
# Version 0.7.13 TG 20000629
#    Added setSearchNavigation & formSearchQuery
# Version 0.7.12 TG 20000623
#    Added loadseltomem
# Version 0.7.11 TG 20000621
#    added loadselects
# Version 0.7.10 IB 20000608
#    corrected the first line
# Version 0.7.9 TG 20000606
#    fixed sortlinks to use the proper fields
# Version 0.7.8 IB 20000301
#    updated copyright
# Version 0.7.7 IB 991112
#    adding subroutines string_todbi and string_exdbi to the string_tomysql
#    and string_exmysql
# Version 0.7.6 IB 991110
#    minor cleanup
# Version 0.7.5 RF 990830
#    made string_exmysql sensitive to \\_ stuff
# Version 0.7.4 RF 990830
#    fixed bug in string_tomysql for "_"->"\_"
# Version 0.7.3 RF 990830
#    fixed bug in string_exmysql for "\_"->"_"
# Version 0.7.2 RF 990829
#    added variant sensitivity to sortlinks()
# Version 0.7.1 GF 990829
#    removing quotes from selects in gen_select
# Version 9 SH 990819
#    hack fix of note on sortlinks
# SH Note:  sortlinks needs to be rewritten 
# Version 8 RF 990208
#    corrected use of fsrelativeroot and storecode -> store_sku_code
# Version 7 RF 981211
#    added nicedecimals( $s ):$s - evens up trailing zeros to currency
# Version 6 RF 981208
#    added getformrows( \% ):\% - returns hashref of hashrefs of rows
# Version 5 RF 981127
#    added sortlinks
#    fixed gen_select matching bug
# Version 4 RF 981125
#    modified keylength operator from equality to >= in gen_select
#    added make_hash_htmlsafe - html encodes certain fields
# Version 3 RF 981123
#    added gen_safeselect - generate select w/only one choice
# Version 2 RF 981013
#    added string to/from mysql conversion operations
#    added string to/from html conversion operations
# version 1 RF 981009
#
# 

#
# gen_select - generates an OPTION value [SELECTED] pulldown
#              content when passed "default","key-length","key-value pairs 
#              vertical slash delimited array"
#           
        

## nicedecimals: takes a float and makes it a \d.\d\d rounded currency value
#
sub nicedecimals {
   $i = shift;
   ( $left,$right ) = split /\./, $i;
   my $round_digit = substr($right, 2, 1);
   $right =~ s/^(..).*/$1/;
   $right = sprintf("%-2s", $right);
   $right =~ s/\s/0/g;

   if ($round_digit >= 5 && $right < 99) {
      $right += 1;
   } elsif ($round_digit >= 5 && $right == 99) {
   	  $right = '00';
	  $left += 1;
   }
   
   return sprintf("%d\.%02d", $left, $right);
} #/nicedecimals


## getformrows( \% ): \%->\%  takes a hashref, finds rows
#     and constructs output rows. FORM KEYS for rows are DELETED
#
sub getformrows {
   my( $r,$k,$v,$f,@temp,$rows );
   $f=$_[0];

   $rows={};

   @temp=keys %{$f};
   while (@temp) {
      $k=shift @temp;
      next if ( $k !~ /^row\./ );
      $v = $f->{$k};
      delete( $f->{$k} );
      $r = ( split /\./, $k )[1];    # rowid
      $k =~ s/^row\.$r\.// ;
      $rows->{$r}->{$k} = $v;
   } # done running through primary form data
   
   return $rows;
} #/getformrows


## sortlinks( @ ): @ - take a list of links and sort them 
#
# 0.7.2 - RF changed to support variants
#
sub sortlinks {
   my( @h,@o,@temp,$i,$k,$c );
   @h = @_; 
   @o = ();
   @temp=();

   for ( $i=0; $i<=$#h; $i++ ) {
      $k='0';
      $k=$h[$i]->{'linkwt_use'};
      $k=$h[$i]->{'linkwt_man'} if ( ( ! $k ) and ( $h[$i]->{'linkwt_man'} ) );
      $k=$h[$i]->{'linkwt_sys'} if ( ( ! $k ) and ( $h[$i]->{'linkwt_sys'} ) );
      $k=$h[$i]->{'linkwt_man_inp'} if ( ( ! $k ) and ( $h[$i]->{'linkwt_man_inp'} ) );
      $k='10' if ( ! $k );

      # $c=$h[$i]->{'CELLWIDTH'};   # do not want cellwidth right now
      # When cellwidth is taken into account, will need to modify the sort key
      # for each cellwidth

      $c='0' if ( ! $c );
      # now construct sort key
#      $k = "$c".'.'."$k X$i"; old way for cellwidth
      $k = "$c"."$k X$i";  # bootstrap to make work with 100
      push @temp, $k; 
   } # end of running through key creation for sorting

   sub bywidthweight { $b <=> $a; } # sort numerically descending 
   @temp=sort bywidthweight @temp;

   while ( @temp ) {
      ( $k, $i ) = split /X/, shift @temp;
      push @o, $h[$i] ;
   } # end of assembling new array   
   return @o;

} #/sortlinks


##### make_hash_htmlsafe( \% ):\% - html encode hash entries matching
#        certain fieldnames
sub make_hash_htmlsafe {
   my(%h,@temp,$k,$v,$fieldlist);
   $fieldlist='(TITLE|ALTTITLE|DESCRIPT|BRIEF|KEYWORDS|AUTHOR|HANDLER)';
   %h = %{$_[0]};
   @temp = keys %h;
   while (@temp) {
      shift @temp;
      next if ( /_SELECT$/ ); 
      next if ( ! /$$fieldlist/ );    
      $h{$_} = string_tohtml( $h{$_} );
   } #/end of looling through keys

   return \%h;
} #/make_hash_htmlsafe


##### gen_safeselect - makes deterministic pulldowns
sub gen_safeselect {
   my(@list,$l,$k,$v,$x,$d,$base);
   ( $base,$l,@list ) = @_;
   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( $base eq $k ) {
            $x.= "<OPTION VALUE=\"$k\" ";
            $x.="SELECTED $k = $base";
            $x.="> $v \n";
            $d=0;
         }
      }
   }
   return $x;
} # /gen_safeselect


##### Gen_Select
sub gen_select {
   my(@list,$l,$k,$v,$x,$d,$base);
   local(*FH);
   ( $base,$l,@list ) = @_;

   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
       $x.= "<OPTION VALUE=\"$k\" ";  # GF removed quotes

      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( "$base" eq "$k" ) {
#            $x.="SELECTED $k = $base";
             $x.="SELECTED"; 
            $d=0;
         }
      }
      $x.="> $v \n";
   }
   return $x;
} # /gen_select


sub myhomegen_select {
   my(@list,$l,$k,$v,$x,$d,$base, $m);
   local(*FH);
   ( $base,$l,@list ) = @_;

   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
      $m=$k." | ".$v;
      $x.= "<OPTION VALUE=\"$k\" ";
      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( "$base" eq "$k" ) {
            $x.="SELECTED $k = $base";
            $d=0;
         }
      }
      $x.="> $m \n";
   }
   return $x;
} # /gen_select


############################################################################
#  PURPOSE:   To read in all .sel files specified in @pulldowns and load 
#             them into $NVP for display
#
#  INPUT:     Array of 'PULLDOWN|NUM' where PULLDOWN is the name of the 
#             file before the .sel and NUM is ???
#
#  OUTPUT:    Sets the global $NVP with the pulldown HTML code
#
#  NOTES:     
#
############################################################################

sub loadselects {
    my @pulldowns = @_;

   my ($k,$v,$x,@raw);

   foreach ( @pulldowns) {

      ( $k,$v) = split /\|/, $_;
      open(FH, "<$selskeletonpath/$k.sel");
      @raw=<FH>;
      chomp @raw;
      close(FH);

      # Make the sel_ files work with the current procedure.
      if ($k =~ /sel_/) {
	  $k =~ s/sel_//gms;
	  $k = uc($k);
	  if ($debug) {
	      print ERLOG "New pulldown name is $k\n";
	  }
      }
      
      $x=$k."_inp";
      $pre=lc($k."_pre");
      $kt=$k."_SELECT";
      $x = lc($x);    # make it lower case for pulldowns!
      $NVP{$kt}= gen_select( $FORM{$x},$v,@raw );
      $NVP{$pre} = $FORM{$x};
      $FORM{$pre} = $FORM{$x};
   }
}

############################################################################
#  PURPOSE:   To load in the contents of a sel file, with the key being
#             the value of the select HTML object, and the value being
#             the text of the select HTML object.  This can be used to display
#             Active vs 15000-A on forms for example.
#
#  INPUT:     Name of the select file to parse and load
#
#  OUTPUT:    Hash with the keys the value, and the values the text of
#             the sel file.
#
#  NOTES:     
#
############################################################################

sub loadseltomem {
    my $pulldown = $_[0];

    my ($k,$v,@raw, $h);

    open(FH, "<$selskeletonpath/$pulldown.sel");
    @raw=<FH>;
    chomp @raw;
    close(FH);

    for ($i = 0; $i <= $#raw; $i++) {
	($k, $v) = split /\|/, $raw[$i];
	$h->{$k} = $v;
    }

    return $h;
}

##### string_tohtml

sub string_tohtml {
   local( $x );

   $x = $_[0];
   $x =~ s/"/&quot;/g;
   
   return $x;
} # /string_tohtml


##### string_exhtml

sub string_exhtml {
   local( $x );

   $x = $_[0];
   $x =~ s/&quot;/"/g;
   
   return $x;
} # /string_exhtml


##### string_tomysql
#
# 0.7.4-990830 RF - changed to single pass 
#
sub string_tomysql {
   local( $x );

   $x = $_[0];
   $x =~ s/('|"|%|_)/\\$1/g ;

   return $x;
} # /string_tomysql


##### string_exmysql
#
# 0.7.5-990830 RF - added preliminary pass to kill \\_ stuff
# 0.7.3-990830 RF - changed to single pass substitution
#
sub string_exmysql {
   local( $x );

   $x = $_[0];
   $x =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
   $x =~ s/\\('|"|%|_|t|n|r)/$1/g ;

   return $x;
} # /string_exmysql


##### string_todbi
#
# 0.7.7-991112 IB - added string_todbi to eventually replace string_tomysql
#
sub string_todbi {
   local( $x );

   $x = $_[0];
#   $x =~ s/('|"|%|_)/\\$1/g ;
   $x =~ s/("|%|_)/\\$1/g ;
   $x =~ s/'/''/g;
   return $x;
} # /string_todbi


##### string_exdbi
#
# 0.7.7-991112 IB - added string_exdbi to eventually replace string_exmysql 
#
sub string_exdbi {
   local( $x );

   $x = $_[0];
   $x =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
   $x =~ s/\\('|"|%|_|t|n|r)/$1/g ;

   return $x;
} # /string_exdbi

############################################################################
# PURPOSE     At the time of calling this routine, we have matched a variant
#             encoding to a search word (both of which are passed in).  
#             Search for all master records that have the encoding as part
#             of their axisname.  
#             Foreach master record:
#                  Get the location in the variant axisname that the 
#                    encoding is used.
#                  Get the code that is used by the encoding body to
#                    represent the search word that was matched.
#                  Find all of the variant products for the master
#                  Foreach variant record:
#                     Compare the code in the location of the vcode, 
#                       of the variant, if it matches, add the record
#
# INPUT       $h - hash of an extraprop encodings record that matched search
#             $search_word - word to search the encodings body for
#             $dbh - open database handler
#
# OUTPUT      $numRowsAdded - The number of rows added during the match
#             $numBind - The number of binded SQL queries executed
#             $numQueries - The number of SQL queries executed
#
# NOTES       The routine which calls this library routine MUST have a
#             locally defined addResult routine that adds the result in
#             the format the the rest of the code requires.  Eventually,
#             all search methods will be the same, and the addResult
#             can be a generic routine added in this library.
#
#             addResult must have input:  $h - hash to add
#                                         $table - table the result is from
############################################################################
sub searchVariantEncodings {
    my ($h) = $_[0];
    my ($search_word) = $_[1];
    my ($dbh) = $_[2];

    my ($codes, $q2, $q2num, $q3, $q3num, $h3, @masters, $master, $x, $codematch, @vcodes, $objtype);
    my ($numRowsAdded, $numBind, $numQueries);

    $numRowsAdded = 0;
    $numBind = 0;
    $numQueries = 0;
    $location = "";
    $codes = "";

    $q2 = "SELECT \* FROM things WHERE axisnames LIKE '\%$h->{'objid'}\%'";
    $q2num = &query($q2, $dbh, \@masters);
    $numQueries++;
    if ($debug) {
	print ERLOG "$q2\n$q2num rows returned\n";  
    }
    
    if ($q2num == 0) {
#	$numr++;
	$numRowsAdded++;
    }

    for ($i = 0; $i < $q2num; $i++) {
	$master = $masters[$i];
	$master->{'searchVariants'} = 0;
	$axisnames = $master->{'axisnames'};
	@ind_axis = split /:/, $axisnames;
	foreach $axis (@ind_axis) {
	    if ($axis eq $h->{'objid'}) {
		$master->{'searchVariants'} = 1;
	    }
	}
	$masters[$i] = $master;
    }
    
    $q3 = "SELECT \* FROM things WHERE ref_from = ?";
    $sth3=$dbh->prepare( $q3 );

    foreach $master (@masters) {
	if ($master->{'searchVariants'} == 1) {
	    if ($debug) {
		print ERLOG "Servicing master record $master->{'objid'}\n";
	    }
	    $location = &getVariantLocation($master, $h->{'objid'});
	    next if ($location eq "");
	    $codes = &getVariantCode($h, $search_word);
	    next if ($codes eq "");
	    $sth3->execute($master->{'objid'});
	    $numBind++;
	    $q3num=0;
	    while ( $rhf = $sth3->fetchrow_hashref(NAME_lc) ) {
		$q3num++;
	    }
	    $sth3->execute($master->{'objid'});
#	$q3num=$sth3->rows;
	    if ($debug) {
		print ERLOG "$q3\n$q3num rows returned\n";
	    }
	    for ($x = 0; $x < $q3num; $x++) {
		$h3 = $sth3->fetchrow_hashref(NAME_lc);
		@vcodes = split /:/, $h3->{'vcode'};
		$codematch = $vcodes[$location];
		if ($debug) {
		    print ERLOG "Checking variant $h3->{'objid'}, $h3->{'vcode'}, $codematch vs $codes\n";
		}
		if ($codes !~ /$codematch/) {
#                      $numr++;
		    $numRowsAdded++;
		    next;
		}
		else {
		    $objtype = $h3->{'objtype'};
		    if ($debug) {
			print ERLOG "\tAdded variant $h3->{'objid'}\n";
		    }
		    &addResult($h3, "things", $objtype);
		}
	    }
	}
    }
    return ($numRowsAdded, $numBind, $numQueries);
}

############################################################################
# PURPOSE     To add all of the variant products for master products
#             that have matched the search criteria in one form or another.
#             This is because the variant products inherit all searchable
#             fields from the master product
#
# INPUT        $dbh - open database handler
#              $MasterRec - hash of master records that have matched the
#              search criteria in one form or another
#
# OUTPUT      $numBind - The number of binded SQL queries executed
#
# NOTES       The routine which calls this library routine MUST have a
#             locally defined addResult routine that adds the result in
#             the format the the rest of the code requires.  Eventually,
#             all search methods will be the same, and the addResult
#             can be a generic routine added in this library.
#
#             addResult must have input:  $h - hash to add
#                                         $table - table the result is from
############################################################################
sub addVariantProducts {
    my ($dbh) = shift;
    my ($MasterRec) = shift;

    my ($q4, $sth4, $key, $q4num, $x, $h4, $objtype);

    my ($numBind);

    $numBind = 0;

    $q4 = "SELECT \* FROM things WHERE ref_from = ? and objid <> ?";
    $sth4=$dbh->prepare( $q4 );

    if ($debug) {
	print ERLOG "Finding all variant products for master products that matched the search\n$q4\n";
    } 
    foreach $key (keys %{$MasterRec}) {
	if ($debug) {
	    print ERLOG "Variant to add:  $key\n";
	}
	$sth4->execute($key, $key);
        $numBind++;
#	$q4num=$sth4->rows;
        $q4num=0;
        while ( $rhf = $sth4->fetchrow_hashref(NAME_lc)) {
           $q4num++;
        }
	$sth4->execute($key, $key);
	for ($x = 0; $x < $q4num; $x++) {
	    if ($debug) {
		print ERLOG "$q4num variant products to add to the results\n";
	    }
	    $h4 = $sth4->fetchrow_hashref(NAME_lc);
	    $objtype = $h4->{'objtype'};
	    &addResult($h4, "things", $objtype);
	}
    }
    return ($numBind);
}

############################################################################
# PURPOSE     Search the encoding body field and get the variant code that 
#             matched the search string specified.  
#
# INPUT       $h - hash of an extraprop encodings record that matched search
#             $searchStr - word to search the encodings body for
#
# OUTPUT      $code - the code that matched the search string
#             For example, 
#               If the search was 'red', and the encoding held a line with
#                d:red, 'd' will be returned.
#             If there is more than one location that matched, then it 
#              seperates the matches with :  
#             For example, if a search was 'x', and an encoding held lines 
#              c:x-large, f:xx-large, x:xxx-large, 'c:f:x' would be returned
#
# NOTES
############################################################################
sub getVariantCode {
    my ($h) = $_[0];
    my ($searchStr) = $_[1];

    my ($i, $body, $cd, $val, $row, @rows, $code);

    if ($debug) {
	print ERLOG "In getCode with $h, and $searchStr, fieldname $h->{'fieldname'}, $h->{'body'}\n";
    }
    ($body = $h->{'body'}) =~ s/\r//g;
	@rows = split /\n/, $body;
	foreach $row (@rows) {
	    ($cd, $val) = split /:/,$row;
	    if ($val =~ $searchStr) {
		if ($code ne "") {
		    $code .= ":$cd";
		}
		else {
		    $code = $cd;
		}
	    }
	}

    if ($debug) {
	print ERLOG "Set code to $code in getCode!\n";
    }
    return $code;
}

############################################################################
# PURPOSE     Search through the master products axisnames, and determine
#             in what location the encoding is located.
#
# INPUT       $h - hash of master product
#             $objid - objid of the encoding to look for in axisnames
#
# OUTPUT      $location - location of the encoding in the axisnames
#             For example:
#                $objid = '17-AA', $h->{'axisnames'} = '2-AA:17-AA:4-AA'
#                  1 will be returned, as the 1st position holds 17-AA
#                  (0 holds 2-AA, and 2 holds 4-AA)
#
# NOTES       If the same encoding is used more than once, it will not
#             find the second one.  It doesn't make sense to include the
#             encoding in more than one location though.
############################################################################
sub getVariantLocation {
    my ($h) = $_[0];
    my ($objid) = $_[1];

    my ($i, $body, $code, $val, $row, @rows, $location);

    if ($debug) {
	print ERLOG "In getLocation with $h, and $objid, axisnames  $h->{'axisnames'}\n";
    }

    $axisnames = $h->{'axisnames'};

    @objids = split /:/, $axisnames;
    
    for ($i = 0; $i <= $#objids; $i++) {
	if ($objids[$i] eq $objid) {
	    $location = $i;
	}

    }
    if ($debug) {
	print ERLOG "Set $location locations in getLocation!\n";
    }
    return $location;
}

############################################################################
# PURPOSE     To set the Next and Back buttons on a search page
#
# INPUT       $pr - location substring:
#                  sid=$sid&mode=$mode&skuskel=$objid&status_inp=$status_inp
#             $foundcount - total number of rows returned from the query
#             $maxrows - maximum number of rows to display on one page
#             $startrow - row to start displaying
#             $lastrow - last row to display
#
# OUTPUT      $back - the back link
#             $next - the next link
#             $firstrecord - the starting row for this page
#
# NOTES
############################################################################
sub setSearchNavigation {
    my ($pr, $foundcount, $maxrows, $startrow, $lastrow) = @_;
    
    my ($back, $next, $firstrecord, $this_start, $this_page, $startPages, $endPage, $numPages, $totalPages);

    $numDisplayPages = 3;     # Total number of pages to show on each side of the pages link from the
                       # current page
    $totalPages = $numDisplayPages * 2 + 3;    # include the current page, first page and last page
    $this_start = $startrow;   
    $this_page = $startrow/$maxrows;
    
    if ($foundcount == 0) {
	$firstrecord = 0; 
    } 
    else {
	$firstrecord = $startrow+1; 
    }
    
    if ($startrow > 0) {
	$startrow = $startrow - $maxrows;
	if ($startrow < 0) {
	    $startrow = 0; 
	}
	$back="<A HREF=\"$scriptpath/$scriptname\?$pr";
	$back.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Back</FONT></A>&nbsp;&nbsp;"; 
    }
    
    if ($lastrow < $foundcount) {
	$startrow = $lastrow;
	$next="\n\n<A HREF=\"$scriptpath/$scriptname\?$pr";
	$next.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">Next</FONT></A>"; 
    }

    $numPages = $foundcount / $maxrows;

    ($numPages, $dec) = split /\./, $numPages;

    if ($dec) {
	$numPages += 1;
    }

    $lastChange = $numPages-$numDisplayPages;
    print ERLOG "Last change is $lastChange, this page is $this_page, total pages is $totalPages\n";

    if ($numPages > 1) {

	if ($numPages > $totalPages) {
	    print ERLOG "Have more than TotalPages, so set last row!\n";
	    if ($this_page >= $lastChange) {
		$startPage = $numPages - (($numDisplayPages * 2) + 1);
		$endPage = $numPages;
	    }
	    elsif ($this_page >=  $numDisplayPages) {
		$startPage = $this_page - $numDisplayPages;   
		$endPage = $this_page + $numDisplayPages;   
	    }
	    else {
		$startPage = 0;
		$endPage = $numDisplayPages * 2 + 1;
	    }

	}
	else {
	    $startPage = 0;
	    $endPage = $numPages;
	}

	if ($debug) {
	    print ERLOG "startPage is $startPage, end is $endPage\n";
	}

	for ($i = 0; $i < $numPages; $i++) {

	    if ((($i >= $startPage) && ($i <= $endPage)) || ($i == ($numPages-1)) || ($i == 0)) {
		$page = $i + 1;
		$startrow = $i * $maxrows;
		$pages.="\n\n<A HREF=\"$scriptpath/$scriptname\?$pr";
		$pages.="&startrow=$startrow\"><FONT FACE=\"arial,helvetica\" SIZE=\"-1\">$page</FONT></A>"; 
		# put spaces after 1st row and before last row
	    }
	    if ( ($i == 0) || ($i == ($numPages - 2)) ){
		$pages .= "&nbsp&nbsp&nbsp&nbsp";
	    }

	}
    }
    return ($back, $next, $pages, $firstrecord);
}

############################################################################
# PURPOSE     To formulate a search query
#
# ALGORITHM   If you enter more than one criteria in the search form the
#             query if formuated by AND
#
#             Object searches for a all objects LIKE the inputted object
#             string in the objid field that are a product class
#
#             Keywords searches title for objects LIKE the inputted keywords
#                    OR brief LIKE inputted keywords
#                    OR keywords like inputted keywords
#             
#             Status searches for all objects that have the inputted status
#             in the things table.
#
# INPUT       $objid - a specific object id to search for
#             $keywords - specific keywords to search for
#             $status - specific status to search for
#             $fields - additional fields to search for
#             $whereclause - additional conditions to add to the where clause
#
# OUTPUT      $q - the formulated query
#
# NOTES
############################################################################
sub formSearchQuery {
    my ($objid, $keyword, $status, $objtype, $fields, $whereclause) = @_;
    my ($q, $flag);

    $q = "SELECT objid, title ";

    if ($fields ne '') {
	$q .= ", $fields ";
    }

    $flag = 0;

    $q .= "FROM things WHERE ";

    if ($objid ne '') {
	$q .= "objid like \'\%$objid\%\'";
	$flag = 1;
    }

    if ($flag) {
	$q .= " AND ";
	$flag = 0;
    }

    if ($keyword ne '') {
	$q .= "(title like \'\%$keyword\%\'";
	$q .= " OR keywords like \'\%$keyword\%\'";
	$q .= " OR brief like \'\%$keyword\%\')";
	$flag = 1;
    }

    if ($flag) {
	$q .= " AND ";
	$flag = 0;
    }

    if ($objtype ne '') {
	$q .= "objtype = '$objtype'";
	$flag = 1;

    }

    if ($flag) {
	$q .= " AND ";
	$flag = 0;
    }

    if ($whereclause ne '') {

	$q .= " $whereclause";
	$flag = 1;
    }

    if ($flag) {
	$q .= " AND ";
	$flag = 0;
    }


    $q .= "status = \'$status\'";

    return ($q);
}

1;
