#!/usr/bin/perl 
#
# localization.pm
#
# Vesrion 15 - 20000522 JN
#    added sel support
# Version 14 - IB 20000605
#    a message from localize_template
# Version 13 - 20000419 RF
#    moved template prefix extractor outside of language loop in loc_templ
#    formatting routines removed
# Version 12 - 20000418 RF
#    fixed defective include syntax in permute_include()
# Version 11 - 20000418 RF
#    moved hash definition back inside loop defining $k for localize_template()
# Version 10 - 20000417 RF
#    added permute_include() 
#    moved static hash definition outside of loop in localize_template()
# Version 9 - 20000417 RF
#    changed localize_template() to load includes from html/{lang}/include 
#       if file starts with the "inc_" string (still falls back to subroot 
#       if no match).
#    changed the tree under bits {lang}/site -> {lang}/html
# Version 8 - 20000417 JC
#    included the loading of include templates
# Version 7 - 20000414 JC
#    corrected $om_file and $om_dir
# Version 6 - 20000409 RF
#    additional work on localize_messages
#    new localize_template to replace load_template
# Version 5 - 20000406 RF
#    completion of new version of om_iso_639
#    correction to message loading routine hash constructor
#    commented out incomplete localization for dates
# Version 4 - 20000406 RF
#    trivial version
# Version 3 - 20000406 RF
#    major changes to support the revised localization and language schema
#    added localize_messages() for loading of language-specific messages
# Version 2 - 20000227 RF
#    added hash keys expected for i/o
#       CHARSET - ISO character set of i/o
#       LANGUAGE - ISO language code of i/o
#       DATE_FORMAT - string identifying data format
#       CURRENCY - string identifying currency
# Version 1 - 20000216 RF
#    created as centralization of all localization-specific I/O routines
#    specifically: 
#       localize_inp_date  - takes a Y-M-D string (format from localization)
#                            returns time in seconds since epoch()
#       localize_inp_money - takes a currency string (format from localization)
#                            returns a real number (decimal currency)
#       localize_out_date -  takes seconds since epoch
#                            returns localization specific human readable date
#       localize_out_money - 
#
# Note - all functions (take?) and return hashrefs!
#         
#

### permute_include($string,$language,$subtree,$error_level):$s
#
# takes a loaded template and finds all references to includes
# of static files (file|virtual), constructs a list of them,
# locates and loads them, then replaces them into the 
# origional template
#
# makes calls to localize_template() in this library file
# 
# Version 1 - 20000417 RF
#
sub permute_include {
   # make certain that this routine doesn't squash variables
   # in the calling script...
   #
   my($k,@temp,$om_render_lang,$verbose_errors,$template,$ok_to_load);
   my($subtree);

   # get the arguments
   ($template,$om_render_lang,$subtree,$verbose_errors) = @_;

   @temp = ( $template =~ /(<!--#include\s+(virtual|file)="(\S+)" *-->)/gis ); 

   foreach $k ( @temp ) {
      next if ( $k !~ /^</ );      # skip if this isn't an intact include
      $f = ( $k =~ /("\S+)/i )[0]; # get the filename...
    
      @temp2 = split /\//, $f;     # in case it contains path data...

      $f = $temp2[$#temp2];        # get the last bit of the path 
      $f = ( split /\./, $f )[0];  # just want what's before the file exten  

      $s = localize_template($f,om_render_lang,$subtree,$verbose_errors);

      $template =~ s/$k/$s/gs;     # put the expanded include in

   } #/foreach reference to an include in the html

   return $template;

} #/permute_include



### localize_messages($scriptname,$lang,$error_level):\%
# 
# takes a script or function name, an ISO 639-3 language code, and error
# message verbosity level --> returns a hashref of hashes 
#
# utilizes global $site hashref for language fallback
#
# Version 1 - 20000406 RF
#
sub localize_messages {
   # make certain that this routine doesn't squash variables
   # in the calling script...
   #
   my($k,$o,$om_render_lang,$verbose_errors,$scriptname,$ok_to_load);
   my($om_dir,$om_file);

   # get the arguments
   ($scriptname,$om_render_lang,$verbose_errors) = @_;

   $o={};     # empty output hashref

   foreach $k ( "errors","messages" ) {

      # skip loading system error messages if diagnostics off
      next if (( ! $verbose_errors ) and ( $k eq "errors" ));

      $om_file="admin_obj.$om_render_lang.$k";
      $om_dir="$site->{'fsrootweb'}/bits/$om_render_lang/system";
      $ok_to_load=1;                      # set true by default

      # check that the render language error messages file exists
      if ( ! -f "$om_dir/$om_file" ) {

         # just try the default language path if that didn't work
         $om_file="admin_obj.$site->{'om_locale_lang_def'}.$k";
         $om_dir="$site->{'fsrootweb'}/bits/$site->{'om_locale_lang_def'}/system";

         $ok_to_load=0 if ( ! -f "$om_dir/$om_file" );

      } #/if the active language error messages exist

      # now have the path and filename to load, if $ok_to_load
      # so load the messages
      if ( $ok_to_load ) {

         open( EM, "<$om_dir/$om_file");
         while (<EM>) {
            chomp;
            next if /^\#/;
            next if ( ! $_ );

            @temp=split /\t/;
            $o->{$k}{$temp[0]}=$temp[1];

         } # done reading the lines of the messages file
         close(EM);

      } #/if - done loading messages, if applicable
   } # /foreach - done loading language-specific messages

   return $o;

} #/localize_messages


### localize_template($template_name,$language,(admin|site),$error_level):$s
# 
# takes a template name w/o extensions, an ISO 639-3 language code, a flag
# indicating the subtree of bits/{lang}/ to look in (nominally "admin"
# for the administrative interfaces, "site" for the HTML interface, "wap"
# for wireless PDAs and such, or "XML" for the XML processes
# returns the template as a single string
#
# utilizes global $site hashref for language fallback
#
# Version 1 - 20000409 RF
#
sub localize_template {
   # make certain that this routine doesn't squash variables
   # in the calling script...
   #
   my($k,$o,$om_render_lang,$verbose_errors,$template,$ok_to_load);
   my($subtree,$om_dir,$om_file,$f,%fh);

   # get the arguments
   ($template,$om_render_lang,$subtree,$verbose_errors) = @_;

   $o="";     # empty output string

   ## loop on languages, starting with the requested language, then
   #  fallback to the default language, then fallback to other 
   #  languages in the active languages list
   #  if all else fails, try "eng" as tree of last resort
   #

   # mapping of abbreviations to trees to search
   #
   $ok_to_load=0;      # set to false by default

   # pull the template prefix and see if it matches defined naming
   # conventions...
   # 
   # TBD - generalize "known prefixes" so arbitrary patterns
   # can be used instead of these hardcodes
   #
   $f = ( $template =~ /^(page_|cell_|inc_|sel_)/ )[0];
   @temp = split /,/, $site->{'om_locale_lang_act'};
   foreach $k ( $om_render_lang,$site->{'om_locale_lang_def'},@temp,"eng" ) {

      next if ( $ok_to_load );   # skip all remaining $k once this is true

      $om_file="$template.txt";
      $om_dir="$site->{'fsrootweb'}/$site->{'fsrelativeroot'}";

      # watch for "cells" and "pages" subtrees in at least 
      # the "html" and "admin" subtrees, and note that includes live
      # in the webserver tree, not /bits!
      # 
      if ( $f ) {
         # for the moment assume this flags the existence of subtrees
         # and add suffixes to the $om_dir as appropriate
         #

         %fh = (   "page_","/bits/$k/$subtree/pages",
                   "cell_","/bits/$k/$subtree/cells",
                   "inc_","/$subtree/$k/include",
		   "sel_","/bits/$k/$subtree/sel" );  
 
         if ( $fh{$f} ) {            # use the path in the hash if defined
            $om_dir.=$fh{$f};        # 
         } else {                    # otherwise default to /bits/{lang}
            $om_dir.="/bits/$k";
         } #/if 

         # account for include files ending in ".html" instead of ".txt"
         $om_file="$template.html" if ( $f eq "inc_" );

      } #/if - done checking for additional tree complexity

      # check that the sought template file exists
      $ok_to_load=1 if ( -f "$om_dir/$om_file" ); 

   } # /foreach - done trying possible fallbacks

   # now have the path and filename to load, if $ok_to_load
   # so load the template
   if ( $ok_to_load ) {

       open( EM, "<$om_dir/$om_file");
      $o = join "",(<EM>);
      close(EM);
   } else {
      $o = "Missing template: path=$om_dir <br> template=$om_file\n";
   } #/if - done loading template, if it or a fallback located

   return $o;

} #/localize_templates


### localize_inp_date(\%):\%
# 
# takes a date input string or parameter set and returns a
# time string as number of seconds since epoch
#
# Deleted RF library version 13-20000419 
# Version 1 - 20000216 RF
#


### localize_out_date(\%):\%
# 
# takes a time() string or null (in which case uses output of
# time() ) and returns a formatted date or date/time string
#
# Deleted RF library version 13-20000419 
# Version 1 - 20000306
#


## om_iso_639
#     args: code/plaintext to map [, code for plaintext]
#     returns: string, tab separated, first field ISO-639-2 "preferred"
#
#       version 2 RF 20000406
#          changed to single hash assignments
#       version 1 JC 20000403
#
sub om_iso_639 {
   my($request,%languages,$om_render_lang,$req_lc,$rv);

   $request=$_[0];           # raw string to match to a language or code
   $req_lc=$request;         # lowercase version of above

   $om_render_lang="";       # language code for plaintext (optional)
   $om_render_lang=$_[1] if ( $_[1] );

   $rv="";                   # return value

   $req_lc =~ tr/A-Z/a-z/;   # slam to lowercase

   # define a massive hash that contains entries for all 2/3 letter
   # ISO-639 language codes, with associated strings containing
   # the following tab-separated list of datums:
   #    1) preferred 3-letter code for the language
   #    2) preferred 2-letter code for the language
   #    3) one or more deprecated codes (2 or 3 char, tab separated list)
   #    4) last item - English plaintext for language name
   #

   %languages=(

      abk=>"abk	ab		Abkhazian",
      ab=>"abk	ab		Abkhazian",
      ace=>"ace			Achinese",
      ach=>"ach			Acoli",
      ada=>"ada			Adangme",
      aa=>"aar	aa		Afar",
      aar=>"aar	aa		Afar",
      afh=>"afh			Afrihili",
      af=>"afr	af		Afrikaans",
      afr=>"afr	af		Afrikaans",
      afa=>"afa			Afro-Asiatic (Other)",
      aka=>"aka			Akan",
      akk=>"akk			Akkadian",
      alb=>"alb	sq	sqi	Albanian",
      sqi=>"alb	sq	sqi	Albanian",
      sq=>"alb	sq	sqi	Albanian",
      ale=>"ale			Aleut",
      alg=>"alg			Algonquian languages",
      tut=>"tut			Altaic (Other)",
      am=>"amh	am		Amharic",
      amh=>"amh	am		Amharic",
      apa=>"apa			Apache languages",
      ar=>"ara	ar		Arabic",
      ara=>"ara	ar		Arabic",
      arc=>"arc			Aramaic",
      arp=>"arp			Arapaho",
      arn=>"arn			Araucanian",
      arw=>"arw			Arawak",
      arm=>"arm	hy	hye	Armenian",
      hye=>"arm	hy	hye	Armenian",
      hy=>"arm	hy	hye	Armenian",
      art=>"art			Artificial (Other)",
      as=>"asm	as		Assamese",
      asm=>"asm	as		Assamese",
      ath=>"ath			Athapascan languages",
      map=>"map			Austronesian (Other)",
      ava=>"ava			Avaric",
      ave=>"ave			Avestan",
      awa=>"awa			Awadhi",
      ay=>"aym	ay		Aymara",
      aym=>"aym	ay		Aymara",
      az=>"aze	az		Azerbaijani",
      aze=>"aze	az		Azerbaijani",
      nah=>"nah			Aztec",
      ban=>"ban			Balinese",
      bat=>"bat			Baltic (Other)",
      bal=>"bal			Baluchi",
      bam=>"bam			Bambara",
      bai=>"bai			Bamileke languages",
      bad=>"bad			Banda",
      bnt=>"bnt			Bantu (Other)",
      bas=>"bas			Basa",
      bak=>"bak	ba		Bashkir",
      ba=>"bak	ba		Bashkir",
      baq=>"baq	eu	eus	Basque",
      eus=>"baq	eu	eus	Basque",
      eu=>"baq	eu	eus	Basque",
      bej=>"bej			Beja",
      bem=>"bem			Bemba",
      bn=>"ben	bn		Bengali",
      ben=>"ben	bn		Bengali",
      ber=>"ber			Berber (Other)",
      bho=>"bho			Bhojpuri",
      bh=>"bih	bh		Bihari",
      bih=>"bih	bh		Bihari",
      bik=>"bik			Bikol",
      bin=>"bin			Bini",
      bi=>"bis	bi		Bislama",
      bis=>"bis	bi		Bislama",
      bra=>"bra			Braj",
      be=>"bre	be		Breton",
      bre=>"bre	be		Breton",
      bug=>"bug			Buginese",
      bg=>"bul	bg		Bulgarian",
      bul=>"bul	bg		Bulgarian",
      bua=>"bua			Buriat",
      bur=>"bur	my	mya	Burmese",
      mya=>"bur	my	mya	Burmese",
      my=>"bur	my	mya	Burmese",
      be=>"bel	be		Byelorussian",
      bel=>"bel	be		Byelorussian",
      cad=>"cad			Caddo",
      car=>"car			Carib",
      ca=>"cat	ca		Catalan",
      cat=>"cat	ca		Catalan",
      cau=>"cau			Caucasian (Other)",
      ceb=>"ceb			Cebuano",
      cel=>"cel			Celtic (Other)",
      cai=>"cai			Central American Indian (Other)",
      chg=>"chg			Chagatai",
      cha=>"cha			Chamorro",
      che=>"che			Chechen",
      chr=>"chr			Cherokee",
      chy=>"chy			Cheyenne",
      chb=>"chb			Chibcha",
      zh=>"chi	zh	zho	Chinese",
      zho=>"chi	zh	zho	Chinese",
      chi=>"chi	zh	zho	Chinese",
      chn=>"chn			Chinook jargon",
      cho=>"cho			Choctaw",
      chu=>"chu			Church Slavic",
      chv=>"chv			Chuvash",
      cop=>"cop			Coptic",
      cor=>"cor			Cornish",
      co=>"cos	co		Corsican",
      cos=>"cos	co		Corsican",
      cre=>"cre			Cree",
      mus=>"mus			Creek",
      crp=>"crp			Creoles and Pidgins (Other)",
      cpe=>"cpe			Creoles and Pidgins, English-based (Other)",
      cpf=>"cpf			Creoles and Pidgins, French-based (Other)",
      cpp=>"cpp			Creoles and Pidgins, Portuguese-based (Other)",
      cus=>"cus			Cushitic (Other)",
      hrv=>"hrv	hr	scr	Croatian",
      hr=>"hrv	hr	scr	Croatian",
      scr=>"hrv	hr	scr	Croatian",
      ces=>"ces	cs	cze	Czech",
      cs=>"ces	cs	cze	Czech",
      cze=>"ces	cs	cze	Czech",
      dak=>"dak			Dakota",
      da=>"dan	da		Danish",
      dan=>"dan	da		Danish",
      del=>"del			Delaware",
      din=>"din			Dinka",
      div=>"div			Divehi",
      doi=>"doi			Dogri",
      dra=>"dra			Dravidian (Other)",
      dua=>"dua			Duala",
      dut=>"dut	nl	nla	Dutch",
      nla=>"dut	nl	nla	Dutch",
      nl=>"dut	nl	nla	Dutch",
      dum=>"dum			Dutch, Middle (ca. 1050-1350)",
      dyu=>"dyu			Dyula",
      dzo=>"dzo	dz		Dzongkha",
      dz=>"dzo	dz		Dzongkha",
      efi=>"efi			Efik",
      egy=>"egy			Egyptian (Ancient)",
      eka=>"eka			Ekajuk",
      elx=>"elx			Elamite",
      en=>"eng	en		English",
      eng=>"eng	en		English",
      enm=>"enm			English, Middle (ca. 1100-1500)",
      ang=>"ang			English, Old (ca. 450-1100)",
      esk=>"esk			Eskimo (Other)",
      eo=>"epo	eo		Esperanto",
      epo=>"epo	eo		Esperanto",
      et=>"est	et		Estonian",
      est=>"est	et		Estonian",
      ewe=>"ewe			Ewe",
      ewo=>"ewo			Ewondo",
      fan=>"fan			Fang",
      fat=>"fat			Fanti",
      fo=>"fao	fo		Faroese",
      fao=>"fao	fo		Faroese",
      fj=>"fij	fj		Fijian",
      fij=>"fij	fj		Fijian",
      fi=>"fin	fi		Finnish",
      fin=>"fin	fi		Finnish",
      fiu=>"fiu			Finno-Ugrian (Other)",
      fon=>"fon			Fon",
      fr=>"fra	fr	fre	French",
      fra=>"fra	fr	fre	French",
      fre=>"fra	fr	fre	French",
      frm=>"frm			French, Middle (ca. 1400-1600)",
      fro=>"fro			French, Old (842- ca. 1400)",
      fy=>"fry	fy		Frisian",
      fry=>"fry	fy		Frisian",
      ful=>"ful			Fulah",
      gaa=>"gaa			Ga",
      gd=>"gae	gd	gdh	Gaelic (Scots)",
      gae=>"gae	gd	gdh	Gaelic (Scots)",
      gdh=>"gae	gd	gdh	Gaelic (Scots)",
      gl=>"glg	gl		Gallegan",
      glg=>"glg	gl		Gallegan",
      lug=>"lug			Ganda",
      gay=>"gay			Gayo",
      gez=>"gez			Geez",
      ka=>"geo	ka	kat	Georgian",
      geo=>"geo	ka	kat	Georgian",
      kat=>"geo	ka	kat	Georgian",
      de=>"deu	de	ger	German",
      deu=>"deu	de	ger	German",
      ger=>"deu	de	ger	German",
      gmh=>"gmh			German, Middle High (ca. 1050-1500)",
      goh=>"goh			German, Old High (ca. 750-1050)",
      gem=>"gem			Germanic (Other)",
      gil=>"gil			Gilbertese",
      gon=>"gon			Gondi",
      got=>"got			Gothic",
      grb=>"grb			Grebo",
      grc=>"grc			Greek, Ancient (to 1453)",
      gre=>"ell	el	gre	Greek, Modern (1453-)",
      ell=>"ell	el	gre	Greek, Modern (1453-)",
      el=>"ell	el	gre	Greek, Modern (1453-)",
      kal=>"kal	kl		Greenlandic",
      kl=>"kal	kl		Greenlandic",
      grn=>"grn	gn		Guarani",
      gn=>"grn	gn		Guarani",
      guj=>"guj	gu		Gujarati",
      gu=>"guj	gu		Gujarati",
      hai=>"hai			Haida",
      ha=>"hau	ha		Hausa",
      hau=>"hau	ha		Hausa",
      haw=>"haw			Hawaiian",
      he=>"heb	he	iw	Hebrew",
      iw=>"heb	he	iw	Hebrew",
      heb=>"heb	he	iw	Hebrew",
      her=>"her			Herero",
      hil=>"hil			Hiligaynon",
      him=>"him			Himachali",
      hi=>"hin	hi		Hindi",
      hin=>"hin	hi		Hindi",
      hmo=>"hmo			Hiri Motu",
      hu=>"hun	hu		Hungarian",
      hun=>"hun	hu		Hungarian",
      hup=>"hup			Hupa",
      iba=>"iba			Iban",
      is=>"ice	is	isl	Icelandic",
      ice=>"ice	is	isl	Icelandic",
      isl=>"ice	is	isl	Icelandic",
      ibo=>"ibo			Igbo",
      ijo=>"ijo			Ijo",
      ilo=>"ilo			Iloko",
      inc=>"inc			Indic (Other)",
      ine=>"ine			Indo-European (Other)",
      id=>"ind	id	in	Indonesian",
      in=>"ind	id	in	Indonesian",
      ind=>"ind	id	in	Indonesian",
      ina=>"ina	ia		Interlingua (International Auxiliary language Association)",
      ia=>"ina	ia		Interlingua (International Auxiliary language Association)",
      ine=>"ine			Indo-European (Other)",
      iu=>"iku	iu		Inuktitut",
      iku=>"iku	iu		Inuktitut",
      ik=>"ipk	ik		Inupiak",
      ipk=>"ipk	ik		Inupiak",
      ira=>"ira			Iranian (Other)",
      gai=>"gai	ga	iri	Irish",
      iri=>"gai	ga	iri	Irish",
      ga=>"gai	ga	iri	Irish",
      sga=>"sga			Irish, Old (to 900)",
      mga=>"mga			Irish, Middle (900 - 1200)",
      iro=>"iro			Iroquoian languages",
      ita=>"ita	it		Italian",
      it=>"ita	it		Italian",
      ja=>"jpn	ja		Japanese",
      jpn=>"jpn	ja		Japanese",
      jav=>"jav	jv	jaw	jw	Javanese",
      jaw=>"jav	jv	jaw	jw	Javanese",
      jv=>"jav	jv	jaw	jw	Javanese",
      jw=>"jav	jv	jaw	jw	Javanese",
      jrb=>"jrb			Judeo-Arabic",
      jpr=>"jpr			Judeo-Persian",
      kab=>"kab			Kabyle",
      kac=>"kac			Kachin",
      kam=>"kam			Kamba",
      kan=>"kan	kn		Kannada",
      kn=>"kan	kn		Kannada",
      kau=>"kau			Kanuri",
      kaa=>"kaa			Kara-Kalpak",
      kar=>"kar			Karen",
      kas=>"kas	ks		Kashmiri",
      ks=>"kas	ks		Kashmiri",
      kaw=>"kaw			Kawi",
      kaz=>"kaz	kk		Kazakh",
      kk=>"kaz	kk		Kazakh",
      kha=>"kha			Khasi",
      khm=>"khm	km		Khmer",
      km=>"khm	km		Khmer",
      khi=>"khi			Khoisan (Other)",
      kho=>"kho			Khotanese",
      kik=>"kik			Kikuyu",
      kin=>"kin	rw		Kinyarwanda",
      rw=>"kin	rw		Kinyarwanda",
      kir=>"kir	ky		Kirghiz",
      ky=>"kir	ky		Kirghiz",
      kom=>"kom			Komi",
      kon=>"kon			Kongo",
      kok=>"kok			Konkani",
      kor=>"kor	ko		Korean",
      ko=>"kor	ko		Korean",
      kpe=>"kpe			Kpelle",
      kro=>"kro			Kru",
      kua=>"kua			Kuanyama",
      kum=>"kum			Kumyk",
      kur=>"kur	ku		Kurdish",
      ku=>"kur	ku		Kurdish",
      kru=>"kru			Kurukh",
      kus=>"kus			Kusaie",
      kut=>"kut			Kutenai",
      lad=>"lad			Ladino",
      lah=>"lah			Lahnda",
      lam=>"lam			Lamba",
      oci=>"oci	oc		Langue d'Oc (post 1500)",
      oc=>"oci	oc		Langue d'Oc (post 1500)",
      lao=>"lao	lo		Lao",
      lao=>"lao	lo		Lao",
      lat=>"lat	la		Latin",
      la=>"lat	la		Latin",
      lv=>"lav	lv		Latvian",
      lat=>"lav	lv		Latvian",
      ltz=>"ltz			Letzeburgesch",
      lez=>"lez			Lezghian",
      lin=>"lin	ln		Lingala",
      ln=>"lin	ln		Lingala",
      lit=>"lit	lt		Lithuanian",
      lt=>"lit	lt		Lithuanian",
      loz=>"loz			Lozi",
      lub=>"lub			Luba-Katanga",
      lui=>"lui			Luiseno",
      lun=>"lun			Lunda",
      luo=>"luo			Luo (Kenya and Tanzania)",
      mac=>"mac	mk	mak	Macedonian",
      mak=>"mac	mk	mak	Macedonian",
      mk=>"mac	mk	mak	Macedonian",
      mad=>"mad			Madurese",
      mag=>"mag			Magahi",
      mai=>"mai			Maithili",
      mak=>"mak			Makasar",
      mlg=>"mlg	mg		Malagasy",
      mg=>"mlg	mg		Malagasy",
      may=>"may	ms	msa	Malay",
      msa=>"may	ms	msa	Malay",
      ms=>"may	ms	msa	Malay",
      mal=>"mal			Malayalam",
      mlt=>"mlt	ml		Maltese",
      ml=>"mlt	ml		Maltese",
      man=>"man			Mandingo",
      mni=>"mni			Manipuri",
      mno=>"mno			Manobo languages",
      max=>"max			Manx",
      mao=>"mao	mi	mri	Maori",
      mri=>"mao	mi	mri	Maori",
      mi=>"mao	mi	mri	Maori",
      mr=>"mar	mr		Marathi",
      mar=>"mar	mr		Marathi",
      chm=>"chm			Mari",
      mah=>"mah			Marshall",
      mwr=>"mwr			Marwari",
      mas=>"mas			Masai",
      myn=>"myn			Mayan languages",
      men=>"men			Mende",
      mic=>"mic			Micmac",
      min=>"min			Minangkabau",
      mis=>"mis			Miscellaneous (Other)",
      moh=>"moh			Mohawk",
      mol=>"mol	mo		Moldavian",
      mo=>"mol	mo		Moldavian",
      mkh=>"mkh			Mon-Kmer (Other)",
      lol=>"lol			Mongo",
      mon=>"mon	mn		Mongolian",
      mn=>"mon	mn		Mongolian",
      mos=>"mos			Mossi",
      mul=>"mul			Multiple languages",
      mun=>"mun			Munda languages",
      nau=>"nau	na		Nauru",
      na=>"nau	na		Nauru",
      nav=>"nav			Navajo",
      nde=>"nde			Ndebele, North",
      nbl=>"nbl			Ndebele, South",
      ndo=>"ndo			Ndongo",
      nep=>"nep	ne		Nepali",
      ne=>"nep	ne		Nepali",
      new=>"new			Newari",
      nic=>"nic			Niger-Kordofanian (Other)",
      ssa=>"ssa			Nilo-Saharan (Other)",
      niu=>"niu			Niuean",
      non=>"non			Norse, Old",
      nai=>"nai			North American Indian (Other)",
      nor=>"nor	no		Norwegian",
      no=>"nor	no		Norwegian",
      nno=>"nno			Norwegian (Nynorsk)",
      nub=>"nub			Nubian languages",
      nym=>"nym			Nyamwezi",
      nya=>"nya			Nyanja",
      nyn=>"nyn			Nyankole",
      nyo=>"nyo			Nyoro",
      nzi=>"nzi			Nzima",
      oji=>"oji			Ojibwa",
      ori=>"ori	or		Oriya",
      or=>"ori	or		Oriya",
      om=>"orm	om		Oromo",
      orm=>"orm	om		Oromo",
      osa=>"osa			Osage",
      oss=>"oss			Ossetic",
      oto=>"oto			Otomian languages",
      pal=>"pal			Pahlavi",
      pau=>"pau			Palauan",
      pli=>"pli			Pali",
      pam=>"pam			Pampanga",
      pag=>"pag			Pangasinan",
      pan=>"pan	pa		Panjabi",
      pa=>"pan	pa		Panjabi",
      pap=>"pap			Papiamento",
      paa=>"paa			Papuan-Australian (Other)",
      fas=>"fas	fa	per	Persian",
      fa=>"fas	fa	per	Persian",
      per=>"fas	fa	per	Persian",
      peo=>"peo			Persian, Old (ca 600 - 400 B.C.)",
      phn=>"phn			Phoenician",
      pol=>"pol	pl		Polish",
      pl=>"pol	pl		Polish",
      pon=>"pon			Ponape",
      por=>"por	pt		Portuguese",
      pt=>"por	pt		Portuguese",
      pra=>"pra			Prakrit languages",
      pro=>"pro			Provencal, Old (to 1500)",
      pus=>"pus	ps		Pushto",
      ps=>"pus	ps		Pushto",
      que=>"que	qu		Quechua",
      qu=>"que	qu		Quechua",
      roh=>"roh	rm		Rhaeto-Romance",
      rm=>"roh	rm		Rhaeto-Romance",
      raj=>"raj			Rajasthani",
      rar=>"rar			Rarotongan",
      roa=>"roa			Romance (Other)",
      ron=>"ron	ro	rum	Romanian",
      ro=>"ron	ro	rum	Romanian",
      rum=>"ron	ro	rum	Romanian",
      rom=>"rom			Romany",
      run=>"run	rn		Rundi",
      rn=>"run	rn		Rundi",
      rus=>"rus	ru		Russian",
      ru=>"rus	ru		Russian",
      sal=>"sal			Salishan languages",
      sam=>"sam			Samaritan Aramaic",
      smi=>"smi			Sami languages",
      smo=>"smo	sm		Samoan",
      sm=>"smo	sm		Samoan",
      sad=>"sad			Sandawe",
      sag=>"sag	sg		Sango",
      sg=>"sag	sg		Sango",
      san=>"san	sa		Sanskrit",
      sa=>"san	sa		Sanskrit",
      srd=>"srd			Sardinian",
      sco=>"sco			Scots",
      sel=>"sel			Selkup",
      sem=>"sem			Semitic (Other)",
      srp=>"srp sr	scc	Serbian",
      sr=>"srp sr	scc	Serbian",
      scc=>"srp sr	scc	Serbian",
      scr=>"scr	sh		Serbo-Croatian",
      sh=>"scr	sh		Serbo-Croatian",
      srr=>"srr			Serer",
      shn=>"shn			Shan",
      sna=>"sna	sn		Shona",
      sn=>"sna	sn		Shona",
      sid=>"sid			Sidamo",
      bla=>"bla			Siksika",
      snd=>"snd	sd		Sindhi",
      sd=>"snd	sd		Sindhi",
      sin=>"sin	si		Singhalese",
      si=>"sin	si		Singhalese",
      sit=>"sit			Sino-Tibetan (Other)",
      sio=>"sio			Siouan languages",
      sla=>"sla			Slavic (Other)",
      ssw=>"ssw	ss		Siswant",
      ss=>"ssw	ss		Siswant",
      slk=>"slk	sk	slo	Slovak",
      sk=>"slk	sk	slo	Slovak",
      slo=>"slk	sk	slo	Slovak",
      slv=>"slv	sl		Slovenian",
      sl=>"slv	sl		Slovenian",
      sog=>"sog			Sogdian",
      som=>"som	so		Somali",
      so=>"som	so		Somali",
      son=>"son			Songhai",
      wen=>"wen			Sorbian languages",
      nso=>"nso			Sotho, Northern ",
      sot=>"sot	st		Sotho, Southern",
      st=>"sot	st		Sotho, Southern",
      sai=>"sai			South American Indian (Other)",
      esl=>"esl	es	spa	Spanish",
      es=>"esl	es	spa	Spanish",
      spa=>"esl	es	spa	Spanish",
      suk=>"suk			Sukuma",
      sux=>"sux			Sumerian",
      sun=>"sun	su		Sudanese",
      sn=>"sun	su		Sudanese",
      sus=>"sus			Susu",
      swa=>"swa	sw		Swahili",
      swa=>"swa	sw		Swahili",
      ssw=>"ssw			Swazi",
      sv=>"sve	sv	swe	Swedish",
      sve=>"sve	sv	swe	Swedish",
      swe=>"sve	sv	swe	Swedish",
      syr=>"syr			Syriac",
      tgl=>"tgl	tl		Tagalog",
      tl=>"tgl	tl		Tagalog",
      tah=>"tah			Tahitian",
      tgk=>"tgk	tg		Tajik",
      tg=>"tgk	tg		Tajik",
      tmk=>"tmh			Tamashek",
      tam=>"tam	ta		Tamil",
      ta=>"tam	ta		Tamil",
      tat=>"tat	tt		Tatar",
      tt=>"tat	tt		Tatar",
      tel=>"tel	te		Telugu",
      te=>"tel	te		Telugu",
      ter=>"ter			Tereno",
      tha=>"tha	th		Thai",
      th=>"tha	th		Thai",
      bod=>"bod	bo	tib	Tibetan",
      bo=>"bod	bo	tib	Tibetan",
      tib=>"bod	bo	tib	Tibetan",
      tig=>"tig			Tigre",
      tir=>"tir	ti		Tigrinya",
      ti=>"tir	ti		Tigrinya",
      tem=>"tem			Timne",
      tiv=>"tiv			Tivi",
      tli=>"tli			Tlingit",
      tog=>"tog	to		Tonga (Nyasa)",
      to=>"tog	to		Tonga (Nyasa)",
      ton=>"ton			Tonga (Tonga Islands)",
      tru=>"tru			Truk",
      tsi=>"tsi			Tsimshian",
      tso=>"tso	ts		Tsonga",
      ts=>"tso	ts		Tsonga",
      tsn=>"tsn	tn		Tswana",
      tn=>"tsn	tn		Tswana",
      tum=>"tum			Tumbuka",
      tur=>"tur	tr		Turkish",
      tr=>"tur	tr		Turkish",
      ota=>"ota			Turkish, Ottoman (1500 - 1928)",
      tuk=>"tuk	tk		Turkmen",
      tk=>"tuk	tk		Turkmen",
      tyv=>"tyv			Tuvinian",
      twi=>"twi	tw		Twi",
      tw=>"twi	tw		Twi",
      uga=>"uga			Ugaritic",
      uig=>"uig	ug		Uighur",
      ug=>"uig	ug		Uighur",
      ukr=>"ukr	uk		Ukrainian",
      uk=>"ukr	uk		Ukrainian",
      umb=>"umb			Umbundu",
      und=>"und			Undetermined",
      urd=>"urd	ur		Urdu",
      ur=>"urd	ur		Urdu",
      uzb=>"uzb	uz		Uzbek",
      uz=>"uzb	uz		Uzbek",
      vai=>"vai			Vai",
      ven=>"ven			Venda",
      vie=>"vie	vi		Vietnamese",
      vi=>"vie	vi		Vietnamese",
      vol=>"vol	vo		Volap&uuml;k",
      vo=>"vol	vo		Volap&uuml;k",
      vot=>"vot			Votic",
      wak=>"wak			Wakashan languages",
      wal=>"wal			Walamo",
      war=>"war			Waray",
      was=>"was			Washo",
      cym=>"cym	cy	wel	Welsh",
      wel=>"cym	cy	wel	Welsh",
      cy=>"cym	cy	wel	Welsh",
      wol=>"wol	wo		Wolof",
      wo=>"wol	wo		Wolof",
      xho=>"xho	xh		Xhosa",
      xh=>"xho	xh		Xhosa",
      sah=>"sah			Yakut",
      yao=>"yao			Yao",
      yap=>"yap			Yap",
      yid=>"yid	yi	ji	Yiddish",
      yi=>"yid	yi	ji	Yiddish",
      ji=>"yid	yi	ji	Yiddish",
      yo=>"yor	yo		Yoruba",
      yor=>"yor	yo		Yoruba",
      zap=>"zap			Zapotec",
      zen=>"zen			Zenaga",
      za=>"zha	za		Zhuang",
      zha=>"zha	za		Zhuang",
      zu=>"zul	zu		Zulu",
      zul=>"zul	zu		Zulu",
      zun=>"zun			Zuni"

   );  # done with defining the major hash

   # there is no error messaging at this time, simply returns null
   # if there is no mapping. This also ignores the plaintext-to-code 
   # problem for the present.
   #

   $rv=$languages{$req_lc};

   if ( ! $rv ) {      # there is no mapping, assume plaintext input

      ## TBD - add a loader for the ISO-639.{lang}.txt file

   } # done attempting to handle plaintext input


   # test to see if we really need the plaintext... did they provide
   # a language for rendering of plaintext strings? Only enter branch
   # if the language mapping has succeeded 
   #
   if (( $om_render_lang ) and ( $rv )) {
      if ( $rv !~ /^eng\t/ ) {

         ## TBD - add a loader for the ISO-639.{lang}.txt file

      } # English is loaded by default (the hashref above), only
      #   need to load a language-specific file if another language

   } # done seeing if we need to load language-sensitive plaintexts


   return $rv;
} #/om_iso_639

1;
