#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# login.pm
#
# Version 0.7.15 JC 20001010
#    fixed login loop problem where
#    substr for $browser gives white
#    space to fill in all 40 characters
# Version 0.7.14 TG 20000710
#    fixed to go to login screen after timeout
# Version 0.7.13 IB 20000628
#    corrected the address
# Version 0.7.10 GF 20000524
#
# Version 0.7.10 JC 20000609
#    deleted ERLOG
# Version 0.7.9 JC 20000330
#    corrected the path for formtools.pm
# Version 0.7.8 IB 20000312
#    corrected == to eq 
#    made the second operand equal the first one in length 
# Version 0.7.7 IB 20000302
#    removed -w
# Version 0.7.6 IB 20000229
#    changed table to database
# Version 0.7.5 IB 20000226
#    change store to site
# Version 0.7.4 GF 20000119
#    removed newsid subroutine
#    added browser-type detection to check_login
#    revised - sid is now handled in cookie
# Version 0.7.3 IB 991205
#    corrected a database call
# Version 0.7.2 IB 991115
#    testmode, database cleanup
# Version 0.7.1 IB 991110
#    corrected a database call
# Version 6 SH 990708
#    totally retrofitted to system 6
#    got rid of adminsessions, now queries in sessions
# Version 5 SH 990618
#    changed the table lookup from table_customer to table
# Version 4 RF 981219
#    converted to store.pm multiple server compliance
# Version 2 RF 981122
#    check_login now uses globals for connect data
# Version 1 RF 981026
#

####
# Library of functions for handling login validation
#
####

###############################################
# Login Validation:
#    check_login 
#       invoked after the form data read but before sending
#       out the http header (it may spew a redirect to the
#       login script).
#
###############################################

# check_login
sub check_login {
   # takes script, mode, SID as args
   # returns $USERNAME if sid for this script and mode, otherwise ""
   #
   local($sc,$m,$sd,$rv,$sti,$num,$rs,$life,$uid,$s,$q,$v,$n,$connect);
   ( $sc,$m,$sd ) = @_;

   $uname="";

   $rv=1;   # ok by default
   $rs="";
   $life=3700;  # number of seconds to live... 

   ## check for a non-zero sid  
   if ( $sd<0 ) { 
      $rv=0;
      $rs="ERROR - there is no session ID $sd";
   }

   if ( $rv ) {
      $connect="$site->{'db_site_config'}";
      $dbu=DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}); 

      ## gather the browser used to login
      #
      $q="SELECT browser from sessions where sid='$sd'";
      $sti=$dbu->prepare( $q ); 
      $sti->execute;
      $browser=$sti->fetchrow;
#      $num=$sti->rows;
      $brlength=length($browser);
      $substr=substr( $ENV{'HTTP_USER_AGENT'},0,$brlength );

    if  ( $browser eq $substr ) {                       # now see if the 
                                                        # browsers match  
     
          $q = "SELECT custid,sid,mdate from sessions where sid=$sd ";
          $sti = $dbu->prepare( $q ); 
          $sti->execute;
#          $num=$sti->rows;

#      if ( $num == 1 ) {                      # OK, have one sid matching
#         ( $session_custid,$session_sid,$session_mdate ) = $sti->fetchrow;
      if ( ( $session_custid,$session_sid,$session_mdate ) = $sti->fetchrow ) {

         $dt=time() - $session_mdate;

         if (  $dt > $life ) {                # login has expired
            $rv=0;
            $rs="ERROR - user logged out! Login inactive more than $life seconds!";
            #/checking mdate 

         } else {                             # update the time of 
                                              # last click

            $num=time();
            $custid=$session_custid;
            $q="UPDATE sessions SET mdate=$num where sid=$sd ";
            $sti = $dbu->do($q);

         } #/done checking freshness of login

      } else {

         # too few or too many records
#         if ( $num ) {
#            $rs="ERROR - there are too many records for $sd";
            # remove bad records for SID

#         } else {
            $rs="ERROR - $sd bad sid! you must log into the system!";
#         }

         $rv=0;
      }

    } else {   # browsers do not match, return error 
      $rs="ERROR - Browsers do not match!"
    } #/browser checking

   } #/looking for sid

   ## now test the uid permissions 
   #
   if ($rv) {

      ## get the user record
      #
      $q="SELECT uname,objtype,status from contacts where ";
      $q.=" objid='$session_custid' ";
      $sti=$dbu->prepare( $q ); 
      $sti->execute;
#      $num=$sti->rows;

      ## verify exactly one row returned
      #
#      if ( $num==1 ) {
#         ($contacts_uname,$contacts_objtype,$contacts_status)=$sti->fetchrow;
      if ( ($contacts_uname,$contacts_objtype,$contacts_status)=$sti->fetchrow ) {

         ## now check permissions
         #
         if ( $contacts_objtype eq '39004-A' ) {
            # this is an employee

            if ( $contacts_status eq '15000-A' ) { 
               # active employee
               $rs=$contacts_uname;

            } else {
               $rs="ERROR - account no longer active";
            } #/ done checking status

         } else {
            # this is not an employee
            $rv="ERROR - this account is not authorized to access this area!"; 
         } #/done with verify 

      } else {
         $rs="ERROR - No accout found for this account!";

      } #/end testing on number of user records in system
   } #/testing user permissions

   ## test whether to redirect back to login screen or not 
   #
   if ( ! $rv ) {
      ## redirect them
      $x = (split /\?/, $ENV{'REQUEST_URI'})[0];  # remove GET stuff
      @temp=split /\//, $x;                        
      pop @temp;                                  # throw away scriptname
      shift(@temp) if ( ! $temp[0] );
      $x= join "/", @temp;                         
      $scriptpath="$x/login.cgi";
      print "Location: /$scriptpath\n\n"; 
   }

   if ( length($rs)<1 ) {
      $rs="ERROR - unknown reason for lack of error message at login";
   }

#   $dbu->disconnect if ($dbu );

   return $rs;

} #/check_login


1;
