#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

##
#    import_shipping_tables.pl
#
# Script to import various format tables of shipping data
# into zonemap and zonerate tables
#
# For present, contains a bunch of file parsers at the
# end of the script, and some variables to select filename
# and type for import.
#
# output is a file suitable for pipeing to, for example, mysql
#
# Version 0.7.11 JC 20000726
#    append dbid files into one big file 
# Version 0.7.10 JC 20000705
#    Correct error in shipping parsing 
# Version 0.7.9 JC 20000629
#    General revision
# Version 0.7.8 JC 20000612
#    Redid FedEx because of their change to PDF
# Version 0.7.7 JC 20000608
#    Refitted for addition of objid, objclass, and objtype
# Version 0.7.6 JC 20000515
#    Now only need to run with files from UPS, FedEx, and USPS 
#    in ./data directory 
# Version 0.7.5 IB 20000302
#    updated copyright
# Version 0.7.4 IB 991115
#    GPL license
# Version 0.7.3 IB 991114
#    deleting some commented out statements
# Version 0.7.2 IB 991113
#    reduce or remove the dependencies on mysql
# Version 0.7.1 IB 991104
#    general cleanup
# Version 4 GF 990830
#    configuration added for myhome   
# Version 4 RF 990527
#    USPS rate files import written
# Version 3 RF 990527
#    USPS zone files import written
# Version 2 RF 990525
#    UPS rate files import written
# Version 1 RF 990525
#    UPS zone file import operational

####
# Initialization
####

use File::Copy;

$now=time();
$zonecount=0;
$ratecount=0;


chdir "./data";

@USPS_zone_files=("default.asp");
@USPS_rate_files=("pm.txt","em.txt");

$UPS = {
	"2da.csv" 	=> "50100-A",
        "2dam.csv" 	=> "50101-A",
        "3dscomm.csv" 	=> "50102-A",
        "3dsres.csv" 	=> "50103-A",
        "gndcomm.csv" 	=> "50104-A",
        "gndres.csv" 	=> "50105-A",
        "xarea.csv" 	=> "50106-A",
        "1da.csv"	=> "50107-A",
        "1dasaver.csv" 	=> "50109-A",
};

$FedEx = { 
          "serv-e2.txt"		=> "50000-A",
          "serv-exp.txt"	=> "50001-A",
          "serv-fo.txt"		=> "50002-A",
	  "serv-freight.txt"	=> "50003-A",
          "serv-po.txt"		=> "50004-A",
          "serv-so.txt"		=> "50005-A",
};

@FedExZfile=("fedexfile1.txt", "fedexfile2.txt");

print "Searching for rate and zone files ...\n\n";

## Get directory listing and look for rate and zone files
#

opendir DIR, "." || die "Can not query directory\n";
@allfiles = readdir DIR;
closedir DIR;

shift @allfiles; # getting rid of .
shift @allfiles; # getting rid of ..

print "Found the following files ... \n @allfiles\n\n" if (@allfiles);
print "No rate and/or zone files ... \n" if (! @allfiles); 

foreach $file (@allfiles) {
   push @csvfiles, $file if ( $file =~ /.csv/ );  
   push @txtfiles, $file if ( $file =~ /.txt/ );
   push @otherfiles, $file if ( $file !~ /.csv/ && $file !~ /.txt/ );
}

## Parse UPS rate and zone files 
#
if (@csvfiles) {
   print "Sorting rate and zone files for United Parcel Service ...\n\n";
   open(RATE, ">UPS.rates.dbid");
   foreach $file (@csvfiles) {
      if ($file =~ /\d\d\d/ && $file !~ /\d\d\d\d\d/) {
      print "Opening UPS zone file $file ...\n";
      open(ZONE, "<$file") || warn "Can not open $file \n";
      @infile=(<ZONE>);
      chomp @infile;
      close(ZONE);
      @lines=@infile;
      @outzone=();
      @special=();

      $zip = $file;
      $zip =~ s/\.csv$//;
      $fromzip = $zip . "00";

      ## format of .csv file data segment is:
      #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
      #     Next Day Air Saver, Next Day Air

      while (( $lines[0] !~ /^00/ ) and ( @lines )) {
         $l = shift @lines;
      } # done chopping header lines

      # read through simple zipcodes
      while (( $lines[0] =~ /^\d\d\d/ ) and ( @lines )) {
         $l = shift @lines ;
         @temp = split /\,/, $l; 
         $zr=shift @temp;
         ($zmin,$zmax) = split /\-/, $zr;
         $zmax=$zmin if ( ! $zmax );
         ## take from 3 digit prefixes to full zips
         $zmin.="00";
         $zmax.="99";
         foreach $i ( 0..$#temp ) {
            next if ( $temp[$i] =~ /\-/ );      # skip null entries
            next if ( ! $temp[$i] );            # skip null entries
            next if ( $temp[$i] =~ /\[\d]/ );   # skip special cases
            # construct the insert statement
            if ( $i == 0 ) {
               $new="INSERT INTO zonemap VALUES ('$zonecount-A','140-A','','50105-A','$fromzip','USA','$zmin','$zmax','USA','$temp[$i]','','$now','') \\g \n"; 
               $zonecount++;
               push @outzone, ( $new );
               $shipmeth="50104";
            } elsif ( $i == 1 ) {
               $new="INSERT INTO zonemap VALUES ('$zonecount-A','140-A','','50103-A','$fromzip','USA','$zmin','$zmax','USA','$temp[$i]','','$now','') \\g \n";
               $zonecount++;
               push @outzone, ( $new ); 
               $shipmeth="50102";
            } elsif ( $i == 2 ) {
               $shipmeth="50100";
            } elsif ( $i == 3 ) {
               $shipmeth="50101";
            } elsif ( $i == 4 ) {
               $shipmeth="50109";
            } else {
               $shipmeth="50107";
            }
            $new = "INSERT INTO zonemap VALUES ";
            $new.="('$zonecount-A','140-A','','$shipmeth-A','$fromzip','USA',";
            $new.="'$zmin','$zmax','USA','$temp[$i]','','$now','') \\g \n";
            $zonecount++;
            push @outzone, ( $new );
         } # done with this zipcode block
      } # done reading through the "normal" portion of the table
   open(OUT, ">UPS.zones.$zip.dbid");
   print OUT @outzone;
   close(OUT);
   }
      next if (! $UPS->{$file});
      print "Renaming rate file $file to $UPS->{$file}.csv ...\n";
      copy($file, "$UPS->{$file}.csv");
#      rename $file, "$UPS->{$file}.csv";
      @outfile=parse_UPS_dom_rates($UPS->{$file});
      while (@outfile) {
      print RATE shift @outfile;
      } # done dumping output array 
   }
   close(RATE);
}
## Parse USPS zone files
#

foreach $uspszonefile (@USPS_zone_files) {
   foreach $file (@otherfiles) {
      if ($file eq $uspszonefile) {
         print "Opening USPS zone file $file ...\n\n"; 
         open(ZONE, "<$file") || warn "Can not open $file \n";
         @lines=(<ZONE>);
         chomp @lines;
         close(ZONE); 
         @output=();
         @special=();
         @outzone=();

         ## format of .asp file data segment is:
         #  <td>zip [.. zip]</td>
         #  <td>zone</td>

         ## preprocess for data segment
         #
         @temp=();
         while ( @lines ) {
            $l = shift @lines;
            if ( $l =~ /ZIP\sCode\sprefix\sis/ ) { 
                $zipinfo=(split /<b>/, $l)[1]; 
                $zip=(split /<\/b/, $zipinfo)[0];
            }
            next if ( $l !~ /<td>.+<\/td>/ );
            $l =~ s/(\s|\*|<td>|<\/td>)+//g ;
            if (length($l)>2) {    # zipcode block
            $l.=":";
            } else {               #
            $l.="\n";
            } # done with zipcode or not
            push @temp, ( $l );
         } # done chewing out relevant bits

         ## Filling out the $fromzip from the file
         #
         $fromzip = $zip . "00";

         ## merge lines
         # 
         $l="";
         while ( @temp ) {
            $li = shift @temp;
            if ( $li =~ /\n/ ) {
               $l.=$li;
               push @lines, ( $l );
            } else {
              $l=$li;
            } #
         } # done merging lines

         ## start writing output file lines
         #
         while ( @lines ) {
            $l = shift @lines;
            chomp $l;
            ( $zips,$zone ) = split /:/, $l;
            ( $zmin,$zmax ) = split /\.\./, $zips;
            $zmax=$zmin if ( ! $zmax );
            $zmax.="99";
            $zmin.="00";

            $new = "INSERT INTO zonemap VALUES ";
            $new.="('$zonecount-A','140-A','','50154-A','$fromzip','USA',";
            $new.="'$zmin','$zmax','USA','$zone','','$now','') \\g \n";
            push @outzone, ( $new ); 
            $zonecount++;
         } # done constructing insert statements

         ## add express domestic as one zone
         $new = "INSERT INTO zonemap VALUES ";
         $new.="('$zonecount-A','140-A','','50151-A','$fromzip','USA',";
         $new.="'00000','99999','USA','0','','$now','') \\g \n";
         push @outzone, ( $new );
         $zonecount++;
      open(OUT, ">USPS.zones.$zip.dbid");
      print OUT @outzone;
      close(OUT);
      # done with parse_USPS_domestic_zones
      }
   }
}
## parse USPS rate files
#
foreach $uspsratefile (@USPS_rate_files) {
   foreach $file (@txtfiles) {
      if ( $uspsratefile eq $file )
      {
         if ( $file eq "pm.txt" ) 
         {
            parse_USPS_dom_rates_priority($file);
         }
         if ( $file eq "em.txt" )
         {
            parse_USPS_dom_rates_express($file);
         }   
      } 
   }
}
open(FH, ">USPS.rates.dbid");
print FH @output;
close(FH); 

## FedEx Zone File sorting
#
# NOTE:  parser is set up to parse zone info via user's cut and paste
#        from PDF file to text file - need to either parse PDF directly
#        or find alternative.

@outzone="";

foreach $file ( @txtfiles ) {
   if ( $file =~ /$FedExZfile[0]/ ) { 
      print "Sorting the FedEx zone file $file ...\n";

      @infile="";
      @zip="";
      @zones="";

      foreach $shipmeth (50000..50005) {

         open(FEDEX, "<fedexfile1.txt");
         @infile=(<FEDEX>);
         chomp @infile;
         close(FEDEX);

         ## Getting the lines with just the zone data in them
         #
         foreach $line (@infile) {
            @zip = split /\s/, $line;
            for ($i=0;$i<$#zip;$i=$i+2) {
               ($zmin,$zmax) = split /\-/, $zip[$i];
               $zmax=$zmin if ( $zmax eq "" );
               $zmin.="00" if ( length($zmin) == 3 );
               $zmax.="99" if ( length($zmax) == 3 );
               $new = "INSERT INTO zonemap VALUES ";
               $new.="('$zonecount-A','140-A','','$shipmeth-A','$fromzip','USA',";
               $new.="'$zmin','$zmax','USA','$zip[$i+1]','','$now','') \\g \n";
               push @outzone, ( $new );
               $zonecount++;
            }
         }
      }
   }

   if ( $file =~ /$FedExZfile[1]/ ) {
      print "Sorting the FedEx zone file $file ...\n";

        @infile=();
        @zip=();
        @zone=();

      foreach $shipmeth (50000..50005) {

        open(FEDEX, "<fedexfile2.txt");
        @infile=(<FEDEX>);
        chomp @infile;
        close(FEDEX);

        ## Getting the lines with just the zone data in them
        #
        foreach $line (@infile) {
           @zip = split /\s/, $line;
           for ($i=0;$i<$#zip;$i=$i+3) {
              ($zmin,$zmax) = split /\-/, $zip[$i];
              $zmax=$zmin if ( $zmax eq "" );   
              $zmin.="00" if ( length($zmin) == 3 );
              $zmax.="99" if ( length($zmax) == 3 );
              $new = "INSERT INTO zonemap VALUES ";
              $new.="('$zonecount-A','140-A','','$shipmeth-A','$fromzip','USA',";
              $new.="'$zmin','$zmax','USA','$zip[$i+1]','','$now','') \\g \n";
              push @outzone, ( $new );
              $zonecount++;
           }
        }
      }
   }
}

open(OUT, ">FedEx.zones.dbid");
print OUT @outzone;
close(OUT);


## FedEx rate charts
#
@output=();
@lines=();
foreach $file (@txtfiles) {
   next if ( ! $FedEx->{$file} ); 
   print "$FedEx->{$file}\n";
   $shipmeth = $FedEx->{$file};
   open(FH, "<$file") || warn "Can not open $file \n";
   @lines=(<FH>);
   chomp @lines;
   close(FH);

   while (( $lines[0] !~ /lb/ ) and ( @lines )) {
      $l = shift @lines;
   } # done chopping header lines

   if ( ! @lines ) { # Test for burn-through
       return ();
   } # done testing for file burn-through

   ## run through remainder of file assuming all lines are same
   #
   $nothing="";

   while ( @lines ) {
      $l = shift @lines;
      $l=~ s/lb\./$nothing/g;
      $l=~ s/lbs\./$nothing/g;
      next if ( $l =~ /\-/ );
      next if ( $l =~ /\,/ );
      @temp = split /\ +/, $l;
      $mass=$temp[1];         # first column is "not to exceed wt"
      next if ( $mass =~ /\+/ );
      next if ( $mass !~ /(\d|\.)+/ ) ;
      $mass.=".00" if ( $mass !~ /\d\./ );
      if ($mass =~ /-/) {
          ($mass1, $mass2) = split /\-/, $mass;
          $mass=$mass2;
          $mass2=~ s/\+/$nothing/g;
      }
      foreach $j ( 1..$#temp ) {
         $l = $temp[$j+1];
         $l =~ tr/0123456789.//cd ;       # kill all non-numerics
         next if ( ! $l );
         next if ( $l =~ /\-/ );
         next if ( $j > 12);
         $zone=$j+1;
         # construct the Insert statements
         if ($zone < 9) {
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','$zone','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
         }
         $ratecount++;
         if ($zone == 9) {
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','9','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++;
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','10','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++;
         }
         if ($zone == 10) {
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','11','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q ); 
             $ratecount++;
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','12','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++;
         } 
         if ($zone == 11) {
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','13','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );   
             $ratecount++;
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','14','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++;
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','15','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++;
             $q = "INSERT INTO zonerate VALUES ";
             $q.= "('$ratecount-A','140-A','','$shipmeth','16','$mass','$l','0.00','','','','$now','') ";
             $q.= " \\g \n";
             push @output, ( $q );
             $ratecount++; 
         }


      } # done running through zones


   } # done with read-through

}
   open(FH, ">FedEx.rates.dbid");
   print FH @output;
   close(FH);

## Put all shipping info into one file (shipping.dbid)
#

`cat *.dbid > shipping.skel`;


### parse_UPS_domestic_rates
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_UPS_dom_rates {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@output,@special,@temp,@zones,$objid,$sfile );   
   $objid = $_[0];
   $sfile = "$objid.csv";
   open(FH, "<$sfile" );
   @lines=(<FH>);
   chomp @infile;
   close(FH);

   @output=();
   @special=();

   ## special parse for xarea file (50106-A)
   #
   if ( $objid eq "50106-A" ) {

      while (( $lines[0] !~ /^(\d\d\d\d\d)/ ) and ( @lines )) {
         $l = shift @lines;
      } # done chopping header lines 

      if ( ! @lines ) { # Test for burn-through
          return ();
      } # done testing for file burn-through 
  
      ## run through remainder of file assuming all lines are same
      #
      while ( @lines ) { 
      $l = shift @lines;   
         @area=split /\s/, $l;
         # construct the insert statement
         $q = "INSERT INTO zonerate VALUES ('$ratecount-A','140-A','','$objid','$area[0]','','1.50','0.00','','','','$now','') \\g \n";
         push @output, ( $q );
         $ratecount++; 
      } 

   }

   ## format of .csv file data segment is:
   #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
   #     Next Day Air Saver, Next Day Air
   if ( $objid ne "50106-A" ) {

      while (( $lines[0] !~ /Exceed/ ) and ( @lines )) {
         $l = shift @lines;
      } # done chopping header lines

      if ( ! @lines ) { # Test for burn-through
          return ();
      } # done testing for file burn-through  
      ## get zone list from this last header line
      #  note column 0 is not a zone
      $l = shift @lines;
      @zones = split /\,/, $l;
      foreach $j ( 1..$#zones ) {
         $zones[$j] =~ tr/0123456789.//cd ;       # kill all non-numerics
      } # done cleaning non-numeric zones

      ## run through remainder of file assuming all lines are same
      #
      while ( @lines ) {
         $l = shift @lines;
         # strip non-numeric non-delimiter characters
         # TBD - RF
         @temp = split /\,/, $l;
         $mass=$temp[0];         # first column is "not to exceed wt"
         next if ( $mass !~ /(\d|\.)+/ ) ;
         $mass.=".00" if ( $mass !~ /\d\./ );
         foreach $j ( 1..$#temp ) {
            $l = $temp[$j];
            $l =~ tr/0123456789.//cd ;       # kill all non-numerics
            next if ( ! $l );
            next if ( $l =~ /\-/ );
            # construct the insert statement
            $q = "INSERT INTO zonerate VALUES ";
            $q.= "('$ratecount-A','140-A','','$objid','$zones[$j]','$mass','$l','0.00','','','','$now','') ";
            $q.= " \\g \n";
            push @output, ( $q );
            $ratecount++;
         } # done running through zones 


      } # done with read-through
   }
   ## TBD - add code here to read through the special cases section
   #        at the end of the file
   return @output;
}# done with parse_UPS_dom_rates  

sub parse_USPS_dom_rates_priority {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@special,@temp,@zones );
   $sfile = $_[0];
   open(FH, "<$sfile" );
   @lines=(<FH>);
   close(FH);
   @output=();
   @special=();

   ## format of .csv file data segment is:
   #  weight, L-3,4,5,6,7,8
   #     Next Day Air Saver, Next Day Air

   while (( $lines[0] !~ /pounds/ ) and ( @lines )) {
      $l = shift @lines;
   } # done chopping header lines
   $l = shift @lines;   # discard last header line

   ## now run through lines and generate insert statements
   #
   while ( @lines ) {
      $l = shift @lines;
      $l =~ s/(\$| )//g ;
      $l =~ s/\s$//g;
      @temp = split /\t/, $l;
      foreach $j ( 2..$#temp ) {   # catch zones 4-8
         $k = $j+2;
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$ratecount-A','140-A','','50154-A','$k','$temp[0]','$temp[$j]','0.00','','','','$now','')";
         $q.= " \\g \n";
         push @output, ( $q );
         $ratecount++;
      } # done with priority zones 4-8
      ## now handle zones 1,2,3
      foreach $j ( 1..3 ) {   # zones L-3 in column 1
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$ratecount-A','140-A','','50154-A','$j','$temp[0]','$temp[1]','0.00','','','','$now','') ";
         $q.= " \\g \n";
         push @output, ( $q );
         $ratecount++;
      } # done with priority zones 1-3

   } # done running through lines
   open(FH, ">USPS.rates.dbid");
   print FH @output;
   close(FH);     
} # end of parse_USPS_dom_rates_priority

sub parse_USPS_dom_rates_express {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@special,@temp,@zones );
   $sfile = $_[0];
   open(FH, "<$sfile" );
   @lines=(<FH>);
   close(FH);
   @special=();

   ## format of .csv file data segment is:
   #  weight, L-3,4,5,6,7,8
   #     Next Day Air Saver, Next Day Air

   while (( $lines[0] !~ /pounds/ ) and ( @lines )) {
      $l = shift @lines;
   } # done chopping header lines
   $l = shift @lines;   # discard last header line

   ## now run through lines and generate insert statements
   #
   while ( @lines ) {
      $l = shift @lines;
      $l =~ s/(\$| )//g ;
      $l =~ s/\s$//g;
      @temp = split /\t/, $l;
      $q = "INSERT INTO zonerate VALUES ";
      $q.= "('$ratecount-A','140-A','','50151-A','0','$temp[0]','$temp[3]','0.00','','','','$now','')";
      $q.= " \\g \n";
      push @output, ( $q );
      $ratecount++;
   } # done running through lines
   open(FH, ">USPS.rates.dbid");
   print FH @output;
   close(FH);
} # end of parse_USPS_dom_rates_express   
1; 
