#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# pqabuilder.pl
#    Program to build PQA (Palm Query Application) files from text files
#    which get converted into HTML files. The PQA (.pqa) file is written
#    to /tmp.
#
# The primary reference: "Palm Query Application Format Specification"
# 3Com (Palm Computing Subsidiary), Revised: 9/99 cjb, DN; 3008-001.
#
# Downloaded from:
# http://www.palmos.com/dev/tech/webclipping/ 
# Web Clipping Resources, WCA/PQA Format, Download the web clipping 
# application format documentation for Windows or Mac OS.
#                                      -------
# Version 1 IB 20000610
#    created the first version
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

require "$relpath/conf/site.pm";
require "$relpath/lib/filetools.pm";
require "$relpath/lib/localization.pm";

$site=setsite();

## multilingual messages
#
$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );
$interface = "palm";

## load messages
#
%script_msg=();
$script=( split /\./, $scriptname )[0];
&message;

$now = time;
%templates=();
%columns=();

$htmlfile="/tmp/pqa" . $now . ".html";

$tn = "page_pqa_home";

&templatecheck( $tn, $om_render_lang, $interface, 0);
$thispage = $templates{$tn};
$t=permutebit( $thispage, $site );
$t=permuteclean( $t );
open( HTML, ">$htmlfile" );
print HTML $t;
close(HTML);

# time offset for creationDate and modificationDate (LastBackupDate)
$timeoffset = 2082844800;  

$comment=0;
$nameflag=0;
$name="";
$metaflag=0;
$meta="";
@fields=();

open( HTML, "<$htmlfile" ) || die "$script_msg{1} \"$htmlfile\".\n";
while ($l=<HTML>) {
   chomp $l;

   next if ( $l !~ /\S/ ); # eliminate blank lines
   if ( $l =~ /<!--/ ) {   # skip comment lines
      $comment=1;
      $comment=0 if ( $l =~ /-->/ );
      next;
   } 
   if ( $comment==1 ) {
      $comment=0 if ( $l =~ /-->/  );
      next;
   }

   if ( $l =~ /<title>/ ) {
      $nameflag=1;
      @fields=split /<title>/, $l;
      $name=$fields[1];
      if ( $name =~ /<\/title>/ ) {
         $nameflag=0;
         @fields=split /<\/title>/, $name;
         $name=$fields[0];
      } else {
         next;
      }
   } elsif ( $l =~ /<TITLE>/ ) {
      $nameflag=1;
      @fields=split /<TITLE>/, $l;
      $name=$fields[1];
      if ( $name =~ /<\/TITLE>/ ) {
         $nameflag=0;
         @fields=split /<\/TITLE>/, $name;
         $name=$fields[0];
      } else {
         next;
      }
   }
   if ( $nameflag==1 ) {
      if ( $l =~ /<\/title>/ ) {
         $nameflag=0;
         @fields=split /<\/title>/, $l;
         $name .= $fields[0];
      } elsif ( $l =~ /<\/TITLE>/ ) {
         $nameflag=0;
         @fields=split /<\/TITLE>/, $l;
         $name .= $fields[0];
      } else { 
         $name .= $l;
         next;
      }
   }
   if ( $l =~ /meta/ ) {
      $metaflag=1;
      @fields=split /<meta /, $l;
      $meta=$fields[1];  
      if ( $meta =~ />/ ) {
         $metaflag=0;
         @fields=split />/, $meta;
         $meta=$fields[0];
      } else {
         next;
      }
   }
   if ( $metaflag==1 ) {
      if ( $l =~ />/ ) {
         $metaflag=0;
         @fields=split />/, $l;
         $meta .= $fields[0];
      } else {
         $meta .= $l;
         next;
      }
   }
}
close(HTML);
unlink $htmlfile;

if ( $meta eq "" or 
   (( $meta ne "" ) and ( $meta !~ /palmcomputingplatform/ ) ) ) {
   print "$script_msg{2} \'palmcomputingplatform.\"\n";
   exit 0;
}

if ( $name eq "" ) {
   print "$script_msg{3}\n";
   exit(0);
} else {
   $n=substr($name,0,1);
   if ( not $n =~ /[a-z]/ and not $n =~ /[A-Z]/ ) {
      print "$script_msg{4}\n";
      exit(0);
   }
}

# cut off suffix with a period
if ( ( $pos=index( $name,'.' ) ) > 0 ) {
   $name=substr( $name,0,$pos )
}

$name =~ tr/A-Za-z0-9_\-//cd;

# limit the length of the file name to 27 characters
$name = substr($name, 0, 27) if ( length($name) > 27 );

$len=length($name);

$basename = $name;

# 1 for null-terminated
$lenbasename = length($basename) + 1;

# round to the nearest word boundary
$lenbasename = int($lenbasename / 2 + (int($lenbasename) % 2) * 0.5); 

$urlstring = "index.html";

$lenurl = length("index.html.html");

$filename .= "/tmp/$name" . ".pqa";

open(PQA, "> $filename") || die "$script_msg{1} \"$filename\".\n";

## PQA Header
#

# header length 32+2+2+9x4 bytes
$hlength = 72;

$numRecords = 1;

$r = {};

# name (32 byte)
$r->{name} = $name;

# attributes (a Word): dmHdrAttrBackup | dmHdrAttrLaunchableData
$r->{attributes} = 0x0208;

# version (a Word): cyrrently 1
$r->{version} = 0x1;

# creationDate (a DWord) 
$r->{ctime} = $now + $timeoffset;

# modificationDate (a DWord)
$r->{mtime} = $now + $timeoffset;

# lastBakupDate (a DWord): initially set to 2
$r->{baktime} = 0x2;

# modificatin number (a DWord): initially set to 0
$r->{modnum} = 0;

# appInfoID (a DWord): 72 - 2 + numRecords x sizeof(RecordEntryType) = 88 
# 70 + numRecords x sizeof(RecordEntryType) = 88
$r->{appinfo_offset} = 88 * $numRecords ; # hexadecimal

# sortInfoID (a DWord): unused in PQA, always 0
$r->{sort_offset} = 0;

# type (a DWord): Always 'pqa ' (0x0x70716120)
$r->{type} = 0x70716120;

# creator (a DWord): Always 'clpr' (0x636c7072)
$r->{creator} = 0x636C7072;

# uniqueIDSeed (a DWord): unused in PQAs, always 0 
$r->{uniqueIDseed} = 0;

$header = pack "a32 n n N N N N N N N N N",
   $r->{name},
   $r->{attributes},
   $r->{version},
   $r->{ctime},
   $r->{mtime},
   $r->{baktime},
   $r->{modnum},
   $r->{appinfo_offset},
   $r->{sort_offset},
   $r->{type},
   $r->{creator},
   $r->{uniqueIDseed};

# write header
print PQA "$header";

## index header
#

$index_header = pack "N n n", 0, $numRecords, 0;

# write index header
print PQA "$index_header";

$rec_offset = 0x00740000;

## index data
#

$index_data = pack "N C C3", $rec_offset,0,0;

# write index data
print PQA "$index_data";

## AppInfo block
#

# signature (a DWord): must be 'lnch' (0x6c6e6368
$r->{signature} = 0x6c6e6368;

# the version of the PQA header (a Word): currently 3
$r->{hdrVersion} = 0x3;

# encVersion (a Word) 
$r->{encVersion} = 0x8001;

# verStrWords (a Word): the length of the verStr (version string) in 16-bit
# words, including any pad byte at the end. Currently 1. 
$r->{verStrWords} = 0x1;

# verStr: a sequence of (verStrWords * 2) bytes; a null-terminated version
# string
$r->{verStr} = 0x3100;

# pqaTitleWords (a Word), the length of the pqaTitleStr that follows, in
# 16-bit words, including any pad byte at the end
$r->{pqaTitleWords} = $lenbasename;

$lenbasename += $lenbasename;

# pqaTitleStr (a Word): a sequence of (pqaTitleWords * 2) bytes; a 
# null-terminated title string 
$r->{pqaTitleStr} = $basename;

# iconWords (a Word): the length of the bitmap data that follows, in
# 16-bit words. including any pad byte at the end. 0 means use a default
# icon.
$r->{iconWords} = 0;

# icon: a sequence of (iconWords * 2) bytes. 0 means use a default.
$r->{icon} = 0;

$appinfo_block = pack "N n n n n n a$lenbasename n n",
   $r->{signature},
   $r->{hdrVersion},
   $r->{encVersion},
   $r->{verStrWords},
   $r->{verStr},
   $r->{pqaTitleWords},
   $r->{pqaTitleStr},
   $r->{iconWords},
   $r->{icon};

# write AppInfo block
print PQA "$appinfo_block";

## web content record
#

# urlOffset (a DWord): always 0x14, a constant
$r->{urlOffset} = 0x14;

# urlLength (a Word): the length of URL string in bytes
$r->{urlLength} = $lenurl;

# round to the nearest word boundary
$lenurl = int($lenurl + (int($lenurl) % 2));

# this PqfWebDocRecordType to the start of the data for this web content
# resource
$r->{dataOffset} = 0x14 + $lenurl;

# dataLength (a Word): the length of the data, in bytes
$r->{dataLength} = 0x94;

$datalen=2 * $r->{dataLength};

# contentType (a Byte): 4 for now (cmlContentTypeTextCml)
$r->{contentType} = 0x4;

# compressionType (a Byte): 1 for now (cmlCompressionTypeBitPacked) 
$r->{compressionType} = 0x1;

# uncompressedDataSize (a DWord): the uncompressed size, in bytes of the
# web content 
$r->{uncompressedDataSize} = 0xc4;

# flags (a Byte): currently unused. It should always be 0.
$r->{flags} = 0;

# reserved (a Byte): currently unused. It should always be 0.
$r->{reserved} = 0;

# URL string
$r->{URLstring} = "index.html.html";

$r->{data} = "1285d5854f381172292004210828942eac2a79c08b914900625754514928b9a144a1756153ce045c8a48949972d5144a068c845c30908c0dce6a23a11788bd2a95045d4aaa783455811752ec117a18e1169dd3117d2422fa9a3212fba67a6a69117218e004a34ab6e42815346400946956dc853920112a55b7214e48024d092444f228949464f8b9a144a6da54442000c21005c4";

$webcontent = pack "N n N n C C N C C a$lenurl H$datalen",
   $r->{urlOffset},
   $r->{urlLength},
   $r->{dataOffset},
   $r->{dataLength},
   $r->{contentType},
   $r->{compressionType},
   $r->{uncompressedDataSize},
   $r->{flags},
   $r->{reserved},
   $r->{URLstring},
   $r->{data};

# write webcontent 
print PQA "$webcontent";
close(PQA);
print "$script_msg{5} $filename.\n";

sub readcols {
   my(@cols,$s);
   $s=$_[0];

   @cols = ( $s =~ m/__column_(\w+?)__/gs );
   return @cols;
} #/readcols

sub templatecheck {
   my($tn,$om_render_lang,$interface,@temp2,$h);

   $tn=$_[0];
   $om_render_lang = $_[1];
   $interface = $_[2];
   # need to load the template
   $templates{$tn}=localize_template( $tn, $om_render_lang, "$interface", 0 );

   @temp2 = readcols($templates{$tn});

   $h={};
   while (@temp2) {
      ( $k,$v ) = ( split /_/, shift @temp2 )[0..1];
      $h->{$k}=$v if ( ! exists ( $h->{$k} ) );
      $h->{$k}=$v if ( $v > $h->{$k} );
   } # done running through columns
   $columns{$tn} = $h;
   return 1;
} #/templatecheck

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

   if ($om_render_lang eq "eng" ) {
      $script_msg{1}="Unable to open";
      $script_msg{2}="The input file does not contain meta statement";
      $script_msg{3}="Output file name is not specified.";
      $script_msg{4}="Output file name does not start with letter.";
      $script_msg{5}="The output was written to the file"; 
   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }
      close(EM);
   }
}

1;
